@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tr-window.c 12997 2011-10-20 00:37:39Z jordan $
+ * $Id: tr-window.c 13195 2012-02-03 21:21:52Z jordan $
  *
  * Copyright (c) Transmission authors and contributors
  *
@@ -733,7 +733,7 @@ gtr_window_new( GtkUIManager * ui_mgr, TrCore * core )
         /* this is to determine the maximum width/height for the label */
         int w=0, h=0;
         PangoLayout * pango_layout;
-        pango_layout = gtk_widget_create_pango_layout( p->ul_lb, "999.99 KiB/s" );
+        pango_layout = gtk_widget_create_pango_layout( p->ul_lb, "999.99 kB/s" );
         pango_layout_get_pixel_size( pango_layout, &w, &h );
         gtk_widget_set_size_request( p->ul_lb, w, h );
         gtk_widget_set_size_request( p->dl_lb, w, h );
