@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2009-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: announcer.h 11280 2010-10-01 13:33:39Z charles $
+ * $Id: announcer.h 12237 2011-03-26 10:22:25Z jordan $
  */
 
 #ifndef __TRANSMISSION__
@@ -35,6 +35,8 @@ typedef enum
 }
 TrackerEventType;
 
+struct tr_pex;
+
 /** @brief Notification object to tell listeners about announce or scrape occurences */
 typedef struct
 {
@@ -46,10 +48,10 @@ typedef struct
     const char * tracker;
 
     /* for TR_TRACKER_PEERS */
-    const uint8_t *  compact;
-    int              compactLen;
+    const struct tr_pex * pex;
+    size_t pexCount;
 
-    /* [0...100] for probability a peer is a seed.  calculated by the leecher/seeder ratio */
+    /* [0...100] for probability a peer is a seed. calculated by the leecher/seeder ratio */
     int8_t           seedProbability;
 }
 tr_tracker_event;
@@ -70,12 +72,11 @@ void tr_announcerClose( tr_session * );
 ***  For torrent customers
 **/
 
-struct tr_torrent_tiers * tr_announcerAddTorrent( struct tr_announcer *,
-                                                  tr_torrent          * torrent,
+struct tr_torrent_tiers * tr_announcerAddTorrent( tr_torrent          * torrent,
                                                   tr_tracker_callback * cb,
                                                   void                * cbdata );
 
-tr_bool tr_announcerHasBacklog( const struct tr_announcer * );
+bool tr_announcerHasBacklog( const struct tr_announcer * );
 
 void tr_announcerResetTorrent( struct tr_announcer*, tr_torrent* );
 
@@ -84,7 +85,7 @@ void tr_announcerRemoveTorrent( struct tr_announcer * ,
 
 void tr_announcerChangeMyPort( tr_torrent * );
 
-tr_bool tr_announcerCanManualAnnounce( const tr_torrent * );
+bool tr_announcerCanManualAnnounce( const tr_torrent * );
 
 void tr_announcerManualAnnounce( tr_torrent * );
 
@@ -103,5 +104,16 @@ tr_tracker_stat * tr_announcerStats( const tr_torrent * torrent,
 void tr_announcerStatsFree( tr_tracker_stat * trackers,
                             int               trackerCount );
 
+/***
+****
+***/
+
+void tr_tracker_udp_upkeep( tr_session * session );
+
+void tr_tracker_udp_close( tr_session * session );
+
+bool tr_tracker_udp_is_idle( const tr_session * session );
+
+
 
 #endif /* _TR_ANNOUNCER_H_ */
