@@ -1,21 +1,21 @@
 /*
- * This file Copyright (C) 2008-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: bencode.c 11445 2010-12-01 04:54:18Z charles $
+ * $Id: bencode.c 13198 2012-02-04 00:34:39Z jordan $
  */
 
 #include <assert.h>
 #include <ctype.h> /* isdigit() */
 #include <errno.h>
 #include <math.h> /* fabs() */
-#include <stdio.h>
-#include <stdlib.h> /* realpath() */
+#include <stdio.h> /* rename() */
+#include <stdlib.h> /* strtoul(), strtod(), realloc(), qsort(), mkstemp() */
 #include <string.h>
 
 #ifdef WIN32 /* tr_mkstemp() */
@@ -24,12 +24,10 @@
  #define _S_IWRITE 128
 #endif
 
-#include <sys/types.h> /* stat() */
-#include <sys/stat.h> /* stat() */
-#include <locale.h>
-#include <unistd.h> /* stat() */
+#include <locale.h> /* setlocale() */
+#include <unistd.h> /* write(), unlink() */
 
-#include <event.h> /* struct evbuffer */
+#include <event2/buffer.h>
 
 #include "ConvertUTF.h"
 
@@ -50,13 +48,13 @@
 ***
 **/
 
-static tr_bool
+static bool
 isContainer( const tr_benc * val )
 {
     return tr_bencIsList( val ) || tr_bencIsDict( val );
 }
 
-static tr_bool
+static bool
 isSomething( const tr_benc * val )
 {
     return isContainer( val ) || tr_bencIsInt( val )
@@ -406,7 +404,7 @@ tr_bencDictFind( tr_benc * val, const char * key )
     return i < 0 ? NULL : &val->val.l.vals[i + 1];
 }
 
-static tr_bool
+static bool
 tr_bencDictFindType( tr_benc * dict, const char * key, int type, tr_benc ** setme )
 {
     return tr_bencIsType( *setme = tr_bencDictFind( dict, key ), type );
@@ -448,11 +446,11 @@ tr_benc_warning( const char * err )
     fprintf( stderr, "warning: %s\n", err );
 }
 
-tr_bool
+bool
 tr_bencGetInt( const tr_benc * val,
                int64_t *       setme )
 {
-    tr_bool success = FALSE;
+    bool success = false;
 
     if( !success && (( success = tr_bencIsInt( val ))))
         if( setme )
@@ -467,10 +465,10 @@ tr_bencGetInt( const tr_benc * val,
     return success;
 }
 
-tr_bool
+bool
 tr_bencGetStr( const tr_benc * val, const char ** setme )
 {
-    const tr_bool success = tr_bencIsString( val );
+    const bool success = tr_bencIsString( val );
 
     if( success )
         *setme = getStr( val );
@@ -478,11 +476,24 @@ tr_bencGetStr( const tr_benc * val, const char ** setme )
     return success;
 }
 
-tr_bool
-tr_bencGetBool( const tr_benc * val, tr_bool * setme )
+bool
+tr_bencGetRaw( const tr_benc * val, const uint8_t ** setme_raw, size_t * setme_len )
+{
+    const bool success = tr_bencIsString( val );
+
+    if( success ) {
+        *setme_raw = (uint8_t*) getStr(val);
+        *setme_len = val->val.s.len;
+    }
+
+    return success;
+}
+
+bool
+tr_bencGetBool( const tr_benc * val, bool * setme )
 {
     const char * str;
-    tr_bool success = FALSE;
+    bool success = false;
 
     if(( success = tr_bencIsBool( val )))
         *setme = val->val.b;
@@ -498,10 +509,10 @@ tr_bencGetBool( const tr_benc * val, tr_bool * setme )
     return success;
 }
 
-tr_bool
+bool
 tr_bencGetReal( const tr_benc * val, double * setme )
 {
-    tr_bool success = FALSE;
+    bool success = false;
 
     if( !success && (( success = tr_bencIsReal( val ))))
         *setme = val->val.d;
@@ -529,57 +540,46 @@ tr_bencGetReal( const tr_benc * val, double * setme )
     return success;
 }
 
-tr_bool
+bool
 tr_bencDictFindInt( tr_benc * dict, const char * key, int64_t * setme )
 {
     return tr_bencGetInt( tr_bencDictFind( dict, key ), setme );
 }
 
-tr_bool
-tr_bencDictFindBool( tr_benc * dict, const char * key, tr_bool * setme )
+bool
+tr_bencDictFindBool( tr_benc * dict, const char * key, bool * setme )
 {
     return tr_bencGetBool( tr_bencDictFind( dict, key ), setme );
 }
 
-tr_bool
+bool
 tr_bencDictFindReal( tr_benc * dict, const char * key, double * setme )
 {
     return tr_bencGetReal( tr_bencDictFind( dict, key ), setme );
 }
 
-tr_bool
+bool
 tr_bencDictFindStr( tr_benc *  dict, const char *  key, const char ** setme )
 {
     return tr_bencGetStr( tr_bencDictFind( dict, key ), setme );
 }
 
-tr_bool
+bool
 tr_bencDictFindList( tr_benc * dict, const char * key, tr_benc ** setme )
 {
     return tr_bencDictFindType( dict, key, TR_TYPE_LIST, setme );
 }
 
-tr_bool
+bool
 tr_bencDictFindDict( tr_benc * dict, const char * key, tr_benc ** setme )
 {
     return tr_bencDictFindType( dict, key, TR_TYPE_DICT, setme );
 }
 
-tr_bool
-tr_bencDictFindRaw( tr_benc         * dict,
-                    const char      * key,
-                    const uint8_t  ** setme_raw,
-                    size_t          * setme_len )
+bool
+tr_bencDictFindRaw( tr_benc * dict, const char * key, const uint8_t  ** setme_raw, size_t * setme_len )
 {
-    tr_benc * child;
-    const tr_bool found = tr_bencDictFindType( dict, key, TR_TYPE_STR, &child );
-
-    if( found ) {
-        *setme_raw = (uint8_t*) getStr(child);
-        *setme_len = child->val.s.len;
-    }
-
-    return found;
+    return tr_bencGetRaw( tr_bencDictFind( dict, key ), setme_raw, setme_len );
 }
 
 /***
@@ -705,7 +705,7 @@ tr_bencListAddReal( tr_benc * list, double val )
 }
 
 tr_benc *
-tr_bencListAddBool( tr_benc * list, tr_bool val )
+tr_bencListAddBool( tr_benc * list, bool val )
 {
     tr_benc * node = tr_bencListAdd( list );
     tr_bencInitBool( node, val );
@@ -799,7 +799,7 @@ tr_bencDictAddInt( tr_benc *    dict,
 }
 
 tr_benc*
-tr_bencDictAddBool( tr_benc * dict, const char * key, tr_bool val )
+tr_bencDictAddBool( tr_benc * dict, const char * key, bool val )
 {
     tr_benc * child = dictFindOrAdd( dict, key, TR_TYPE_BOOL );
     tr_bencInitBool( child, val );
@@ -942,35 +942,44 @@ struct SaveNode
 };
 
 static void
-nodeInitDict( struct SaveNode * node, const tr_benc * val )
+nodeInitDict( struct SaveNode * node, const tr_benc * val, bool sort_dicts )
 {
-    int               i, j;
-    int               nKeys;
-    struct KeyIndex * indices;
+    const int n = val->val.l.count;
+    const int nKeys = n / 2;
 
     assert( tr_bencIsDict( val ) );
 
-    nKeys = val->val.l.count / 2;
     node->val = val;
-    node->children = tr_new0( int, nKeys * 2 );
+    node->children = tr_new0( int, n );
 
-    /* ugh, a dictionary's children have to be sorted by key... */
-    indices = tr_new( struct KeyIndex, nKeys );
-    for( i = j = 0; i < ( nKeys * 2 ); i += 2, ++j )
+    if( sort_dicts )
     {
-        indices[j].key = getStr(&val->val.l.vals[i]);
-        indices[j].index = i;
+        int i, j;
+        struct KeyIndex * indices = tr_new( struct KeyIndex, nKeys );
+        for( i=j=0; i<n; i+=2, ++j )
+        {
+            indices[j].key = getStr(&val->val.l.vals[i]);
+            indices[j].index = i;
+        }
+        qsort( indices, j, sizeof( struct KeyIndex ), compareKeyIndex );
+        for( i = 0; i < j; ++i )
+        {
+            const int index = indices[i].index;
+            node->children[node->childCount++] = index;
+            node->children[node->childCount++] = index + 1;
+        }
+
+        tr_free( indices );
     }
-    qsort( indices, j, sizeof( struct KeyIndex ), compareKeyIndex );
-    for( i = 0; i < j; ++i )
+    else
     {
-        const int index = indices[i].index;
-        node->children[node->childCount++] = index;
-        node->children[node->childCount++] = index + 1;
+        int i;
+
+        for( i=0; i<n; ++i )
+            node->children[node->childCount++] = i;
     }
 
-    assert( node->childCount == nKeys * 2 );
-    tr_free( indices );
+    assert( node->childCount == n );
 }
 
 static void
@@ -997,13 +1006,13 @@ nodeInitLeaf( struct SaveNode * node, const tr_benc * val )
 }
 
 static void
-nodeInit( struct SaveNode * node, const tr_benc * val )
+nodeInit( struct SaveNode * node, const tr_benc * val, bool sort_dicts )
 {
     static const struct SaveNode INIT_NODE = { NULL, 0, 0, 0, NULL };
     *node = INIT_NODE;
 
          if( tr_bencIsList( val ) ) nodeInitList( node, val );
-    else if( tr_bencIsDict( val ) ) nodeInitDict( node, val );
+    else if( tr_bencIsDict( val ) ) nodeInitDict( node, val, sort_dicts );
     else                            nodeInitLeaf( node, val );
 }
 
@@ -1028,13 +1037,14 @@ struct WalkFuncs
 static void
 bencWalk( const tr_benc          * top,
           const struct WalkFuncs * walkFuncs,
-          void                   * user_data )
+          void                   * user_data,
+          bool                     sort_dicts )
 {
     int stackSize = 0;
     int stackAlloc = 64;
     struct SaveNode * stack = tr_new( struct SaveNode, stackAlloc );
 
-    nodeInit( &stack[stackSize++], top );
+    nodeInit( &stack[stackSize++], top, sort_dicts );
 
     while( stackSize > 0 )
     {
@@ -1044,7 +1054,7 @@ bencWalk( const tr_benc          * top,
         if( !node->valIsVisited )
         {
             val = node->val;
-            node->valIsVisited = TRUE;
+            node->valIsVisited = true;
         }
         else if( node->childIndex < node->childCount )
         {
@@ -1086,7 +1096,7 @@ bencWalk( const tr_benc          * top,
                             stackAlloc *= 2;
                             stack = tr_renew( struct SaveNode, stack, stackAlloc );
                         }
-                        nodeInit( &stack[stackSize++], val );
+                        nodeInit( &stack[stackSize++], val, sort_dicts );
                     }
                     break;
 
@@ -1098,7 +1108,7 @@ bencWalk( const tr_benc          * top,
                             stackAlloc *= 2;
                             stack = tr_renew( struct SaveNode, stack, stackAlloc );
                         }
-                        nodeInit( &stack[stackSize++], val );
+                        nodeInit( &stack[stackSize++], val, sort_dicts );
                     }
                     break;
 
@@ -1215,7 +1225,7 @@ void
 tr_bencFree( tr_benc * val )
 {
     if( isSomething( val ) )
-        bencWalk( val, &freeWalkFuncs, NULL );
+        bencWalk( val, &freeWalkFuncs, NULL, false );
 }
 
 /***
@@ -1233,7 +1243,7 @@ struct ParentState
 /** @brief Implementation helper class for tr_bencToBuffer(TR_FMT_JSON) */
 struct jsonWalk
 {
-    tr_bool doIndent;
+    bool doIndent;
     tr_list * parents;
     struct evbuffer *  out;
 };
@@ -1255,7 +1265,7 @@ jsonIndent( struct jsonWalk * data )
 static void
 jsonChildFunc( struct jsonWalk * data )
 {
-    if( data->parents )
+    if( data->parents && data->parents->data )
     {
         struct ParentState * parentState = data->parents->data;
 
@@ -1267,7 +1277,7 @@ jsonChildFunc( struct jsonWalk * data )
                 if( !( i % 2 ) )
                     evbuffer_add( data->out, ": ", data->doIndent ? 2 : 1 );
                 else {
-                    const tr_bool isLast = parentState->childIndex == parentState->childCount;
+                    const bool isLast = parentState->childIndex == parentState->childCount;
                     if( !isLast ) {
                         evbuffer_add( data->out, ", ", data->doIndent ? 2 : 1 );
                         jsonIndent( data );
@@ -1278,7 +1288,7 @@ jsonChildFunc( struct jsonWalk * data )
 
             case TR_TYPE_LIST:
             {
-                const tr_bool isLast = ++parentState->childIndex == parentState->childCount;
+                const bool isLast = ++parentState->childIndex == parentState->childCount;
                 if( !isLast ) {
                     evbuffer_add( data->out, ", ", data->doIndent ? 2 : 1 );
                     jsonIndent( data );
@@ -1355,41 +1365,53 @@ jsonRealFunc( const tr_benc * val, void * vdata )
 static void
 jsonStringFunc( const tr_benc * val, void * vdata )
 {
+    char * out;
+    char * outwalk;
+    char * outend;
+    struct evbuffer_iovec vec[1];
     struct jsonWalk * data = vdata;
     const unsigned char * it = (const unsigned char *) getStr(val);
     const unsigned char * end = it + val->val.s.len;
 
-    evbuffer_expand( data->out, val->val.s.len + 2 );
-    evbuffer_add( data->out, "\"", 1 );
+    evbuffer_reserve_space( data->out, val->val.s.len * 4, vec, 1 );
+    out = vec[0].iov_base;
+    outend = out + vec[0].iov_len;
+
+    outwalk = out;
+    *outwalk++ = '"';
 
     for( ; it!=end; ++it )
     {
         switch( *it )
         {
-            case '\b': evbuffer_add( data->out, "\\b", 2 ); break;
-            case '\f': evbuffer_add( data->out, "\\f", 2 ); break;
-            case '\n': evbuffer_add( data->out, "\\n", 2 ); break;
-            case '\r': evbuffer_add( data->out, "\\r", 2 ); break;
-            case '\t': evbuffer_add( data->out, "\\t", 2 ); break;
-            case '"': evbuffer_add( data->out, "\\\"", 2 ); break;
-            case '\\': evbuffer_add( data->out, "\\\\", 2 ); break;
+            case '\b': *outwalk++ = '\\'; *outwalk++ = 'b'; break;
+            case '\f': *outwalk++ = '\\'; *outwalk++ = 'f'; break;
+            case '\n': *outwalk++ = '\\'; *outwalk++ = 'n'; break;
+            case '\r': *outwalk++ = '\\'; *outwalk++ = 'r'; break;
+            case '\t': *outwalk++ = '\\'; *outwalk++ = 't'; break;
+            case '"' : *outwalk++ = '\\'; *outwalk++ = '"'; break;
+            case '\\': *outwalk++ = '\\'; *outwalk++ = '\\'; break;
 
             default:
                 if( isascii( *it ) )
-                    evbuffer_add( data->out, it, 1 );
+                    *outwalk++ = *it;
                 else {
                     const UTF8 * tmp = it;
-                    UTF32        buf = 0;
-                    UTF32 *      u32 = &buf;
-                    ConversionResult result = ConvertUTF8toUTF32( &tmp, end, &u32, &buf + 1, 0 );
+                    UTF32        buf[1] = { 0 };
+                    UTF32 *      u32 = buf;
+                    ConversionResult result = ConvertUTF8toUTF32( &tmp, end, &u32, buf + 1, 0 );
                     if((( result==conversionOK ) || (result==targetExhausted)) && (tmp!=it)) {
-                        evbuffer_add_printf( data->out, "\\u%04x", (unsigned int)buf );
+                        outwalk += tr_snprintf( outwalk, outend-outwalk, "\\u%04x", (unsigned int)buf[0] );
                         it = tmp - 1;
                     }
                 }
         }
     }
-    evbuffer_add( data->out, "\"", 1 );
+
+    *outwalk++ = '"';
+    vec[0].iov_len = outwalk - out;
+    evbuffer_commit_space( data->out, vec, 1 );
+
     jsonChildFunc( data );
 }
 
@@ -1423,7 +1445,7 @@ jsonContainerEndFunc( const tr_benc * val,
                       void *          vdata )
 {
     struct jsonWalk * data = vdata;
-    int               emptyContainer = FALSE;
+    int               emptyContainer = false;
 
     jsonPopParent( data );
     if( !emptyContainer )
@@ -1457,7 +1479,7 @@ tr_bencListCopy( tr_benc * target, const tr_benc * src )
     {
        if( tr_bencIsBool( val ) )
        {
-           tr_bool boolVal = 0;
+           bool boolVal = 0;
            tr_bencGetBool( val, &boolVal );
            tr_bencListAddBool( target, boolVal );
        }
@@ -1503,10 +1525,10 @@ tr_bencDictSize( const tr_benc * dict )
     return count;
 }
 
-tr_bool
+bool
 tr_bencDictChild( tr_benc * dict, size_t n, const char ** key, tr_benc ** val )
 {
-    tr_bool success = 0;
+    bool success = 0;
 
     assert( tr_bencIsDict( dict ) );
 
@@ -1540,7 +1562,7 @@ tr_bencMergeDicts( tr_benc * target, const tr_benc * source )
         {
             if( tr_bencIsBool( val ) )
             {
-                tr_bool boolVal;
+                bool boolVal;
                 tr_bencGetBool( val, &boolVal );
                 tr_bencDictAddBool( target, key, boolVal );
             }
@@ -1571,6 +1593,16 @@ tr_bencMergeDicts( tr_benc * target, const tr_benc * source )
                     tr_bencListCopy( tr_bencDictAddList( target, key, tr_bencListSize( val ) ), val );
                 }
             }
+            else if( tr_bencIsDict( val ) )
+            {
+                tr_benc * target_dict = tr_bencDictFind( target, key );
+
+                if( target_dict == NULL )
+                    target_dict = tr_bencDictAddDict( target, key, tr_bencDictSize( val ) );
+
+                if( tr_bencIsDict( target_dict ) )
+                    tr_bencMergeDicts( target_dict, val );
+            }
             else
             {
                 tr_dbg( "tr_bencMergeDicts skipping \"%s\"", key );
@@ -1583,16 +1615,17 @@ tr_bencMergeDicts( tr_benc * target, const tr_benc * source )
 ****
 ***/
 
-void
-tr_bencToBuf( const tr_benc * top, tr_fmt_mode mode, struct evbuffer * buf )
+struct evbuffer *
+tr_bencToBuf( const tr_benc * top, tr_fmt_mode mode )
 {
-    evbuffer_drain( buf, EVBUFFER_LENGTH( buf ) );
+    struct evbuffer * buf = evbuffer_new( );
+
     evbuffer_expand( buf, 4096 ); /* alloc a little memory to start off with */
 
     switch( mode )
     {
         case TR_FMT_BENC:
-            bencWalk( top, &saveFuncs, buf );
+            bencWalk( top, &saveFuncs, buf, true );
             break;
 
         case TR_FMT_JSON:
@@ -1601,24 +1634,24 @@ tr_bencToBuf( const tr_benc * top, tr_fmt_mode mode, struct evbuffer * buf )
             data.doIndent = mode==TR_FMT_JSON;
             data.out = buf;
             data.parents = NULL;
-            bencWalk( top, &jsonWalkFuncs, &data );
-            if( EVBUFFER_LENGTH( buf ) )
+            bencWalk( top, &jsonWalkFuncs, &data, true );
+            if( evbuffer_get_length( buf ) )
                 evbuffer_add_printf( buf, "\n" );
             break;
         }
     }
+
+    return buf;
 }
 
 char*
 tr_bencToStr( const tr_benc * top, tr_fmt_mode mode, int * len )
 {
-    char * ret;
-    struct evbuffer * buf = evbuffer_new( );
-    tr_bencToBuf( top, mode, buf );
-    ret = tr_strndup( EVBUFFER_DATA( buf ), EVBUFFER_LENGTH( buf ) );
+    struct evbuffer * buf = tr_bencToBuf( top, mode );
+    const size_t n = evbuffer_get_length( buf );
+    char * ret = evbuffer_free_to_str( buf );
     if( len != NULL )
-        *len = (int) EVBUFFER_LENGTH( buf );
-    evbuffer_free( buf );
+        *len = (int) n;
     return ret;
 }
 
@@ -1636,17 +1669,6 @@ tr_mkstemp( char * template )
 #endif
 }
 
-/* portability wrapper for fsync(). */
-static void
-tr_fsync( int fd )
-{
-#ifdef WIN32
-    _commit( fd );
-#else
-    fsync( fd );
-#endif
-}
-
 int
 tr_bencToFile( const tr_benc * top, tr_fmt_mode mode, const char * filename )
 {
@@ -1663,31 +1685,50 @@ tr_bencToFile( const tr_benc * top, tr_fmt_mode mode, const char * filename )
     /* if the file already exists, try to move it out of the way & keep it as a backup */
     tmp = tr_strdup_printf( "%s.tmp.XXXXXX", filename );
     fd = tr_mkstemp( tmp );
+    tr_set_file_for_single_pass( fd );
     if( fd >= 0 )
     {
-        int len;
-        char * str = tr_bencToStr( top, mode, &len );
+        int nleft;
 
-        if( write( fd, str, len ) == (ssize_t)len )
+        /* save the benc to a temporary file */
         {
-            struct stat sb;
-            const tr_bool already_exists = !stat( filename, &sb ) && S_ISREG( sb.st_mode );
+            struct evbuffer * buf = tr_bencToBuf( top, mode );
+            const char * walk = (const char *) evbuffer_pullup( buf, -1 );
+            nleft = evbuffer_get_length( buf );
 
-            tr_fsync( fd );
+            while( nleft > 0 ) {
+                const int n = write( fd, walk, nleft );
+                if( n >= 0 ) {
+                    nleft -= n;
+                    walk += n;
+                }
+                else if( errno != EAGAIN ) {
+                    err = errno;
+                    break;
+                }
+            }
+
+            evbuffer_free( buf );
+        }
+
+        if( nleft > 0 )
+        {
+            tr_err( _( "Couldn't save temporary file \"%1$s\": %2$s" ), tmp, tr_strerror( err ) );
+            tr_close_file( fd );
+            unlink( tmp );
+        }
+        else
+        {
+            //tr_fsync( fd );
             tr_close_file( fd );
 
-            if( !already_exists || !unlink( filename ) )
+#ifdef WIN32
+            if( MoveFileEx( tmp, filename, MOVEFILE_REPLACE_EXISTING ) )
+#else
+            if( !rename( tmp, filename ) )
+#endif
             {
-                if( !rename( tmp, filename ) )
-                {
-                    tr_inf( _( "Saved \"%s\"" ), filename );
-                }
-                else
-                {
-                    err = errno;
-                    tr_err( _( "Couldn't save file \"%1$s\": %2$s" ), filename, tr_strerror( err ) );
-                    unlink( tmp );
-                }
+                tr_inf( _( "Saved \"%s\"" ), filename );
             }
             else
             {
@@ -1696,15 +1737,6 @@ tr_bencToFile( const tr_benc * top, tr_fmt_mode mode, const char * filename )
                 unlink( tmp );
             }
         }
-        else
-        {
-            err = errno;
-            tr_err( _( "Couldn't save temporary file \"%1$s\": %2$s" ), tmp, tr_strerror( err ) );
-            tr_close_file( fd );
-            unlink( tmp );
-        }
-
-        tr_free( str );
     }
     else
     {
