@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: clients.c 12502 2011-06-16 12:08:52Z livings124 $
+ * $Id: clients.c 13099 2011-11-22 03:30:37Z livings124 $
  */
 
 /* thanks amc1! */
@@ -304,6 +304,29 @@ tr_clientForId( char * buf, size_t buflen, const void * id_in )
             return;
     }
 
+    /* uTorrent will replace the trailing dash with an extra digit for longer version numbers */
+    if( id[0] == '-' )
+    {
+        if( !memcmp( id+1, "UT", 2 ) )
+        {
+            tr_snprintf( buf, buflen, "\xc2\xb5Torrent %d.%d.%d%s",
+                        strint(id+3,1), strint(id+4,1), strint(id+5,2), getMnemonicEnd(id[7]) );
+        }
+        else if( !memcmp( id+1, "UM", 2 ) )
+        {
+            tr_snprintf( buf, buflen, "\xc2\xb5Torrent Mac %d.%d.%d%s",
+                        strint(id+3,1), strint(id+4,1), strint(id+5,2), getMnemonicEnd(id[7]) );
+        }
+        else if( !memcmp( id+1, "UE", 2 ) )
+        {
+            tr_snprintf( buf, buflen, "\xc2\xb5Torrent Embedded %d.%d.%d%s",
+                        strint(id+3,1), strint(id+4,1), strint(id+5,2), getMnemonicEnd(id[7]) );
+        }
+
+        if( *buf )
+            return;
+    }
+
     /* Mainline */
     if( isMainlineStyle( id ) )
     {
