@@ -1,4 +1,5 @@
 #include <stdio.h>
+#include <string.h> /* memset() */
 
 #include "transmission.h"
 #include "history.h"
@@ -32,19 +33,19 @@ static int test = 0;
 static int
 test1( void )
 {
-    tr_recentHistory * h;
+    tr_recentHistory h;
 
-    h = tr_historyNew( 60, 10 );
-    tr_historyAdd( h, 10000, 1 );
-    check( (int)tr_historyGet( h, 12000, 1000 ) == 0 )
-    check( (int)tr_historyGet( h, 12000, 3000 ) == 1 )
-    check( (int)tr_historyGet( h, 12000, 5000 ) == 1 )
-    tr_historyAdd( h, 20000, 1 );
-    check( (int)tr_historyGet( h, 22000,  1000 ) == 0 )
-    check( (int)tr_historyGet( h, 22000,  3000 ) == 1 )
-    check( (int)tr_historyGet( h, 22000, 15000 ) == 2 )
-    check( (int)tr_historyGet( h, 22000, 20000 ) == 2 )
-    tr_historyFree( h );
+    memset( &h, 0, sizeof( tr_recentHistory ) );
+
+    tr_historyAdd( &h, 10000, 1 );
+    check( (int)tr_historyGet( &h, 12000, 1000 ) == 0 )
+    check( (int)tr_historyGet( &h, 12000, 3000 ) == 1 )
+    check( (int)tr_historyGet( &h, 12000, 5000 ) == 1 )
+    tr_historyAdd( &h, 20000, 1 );
+    check( (int)tr_historyGet( &h, 22000,  1000 ) == 0 )
+    check( (int)tr_historyGet( &h, 22000,  3000 ) == 1 )
+    check( (int)tr_historyGet( &h, 22000, 15000 ) == 2 )
+    check( (int)tr_historyGet( &h, 22000, 20000 ) == 2 )
 
     return 0;
 }
