@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: inout.h 11709 2011-01-19 13:48:47Z jordan $
+ * $Id: inout.h 12545 2011-07-13 03:23:37Z jordan $
  */
 
 #ifndef __TRANSMISSION__
@@ -34,11 +34,10 @@ int tr_ioRead( struct tr_torrent   * tor,
                uint32_t              len,
                uint8_t             * setme );
 
-int
-tr_ioPrefetch( tr_torrent       * tor,
-               tr_piece_index_t   pieceIndex,
-               uint32_t           begin,
-               uint32_t           len );
+int tr_ioPrefetch( tr_torrent       * tor,
+                   tr_piece_index_t   pieceIndex,
+                   uint32_t           begin,
+                   uint32_t           len );
 
 /**
  * Writes the block specified by the piece index, offset, and length.
@@ -53,18 +52,18 @@ int tr_ioWrite( struct tr_torrent  * tor,
 /**
  * @brief Test to see if the piece matches its metainfo's SHA1 checksum.
  */
-tr_bool tr_ioTestPiece( tr_torrent       * tor,
-                        tr_piece_index_t   piece );
+bool tr_ioTestPiece( tr_torrent       * tor,
+                     tr_piece_index_t   piece );
 
 
 /**
  * Converts a piece index + offset into a file index + offset.
  */
-void     tr_ioFindFileLocation( const tr_torrent * tor,
-                                tr_piece_index_t   pieceIndex,
-                                uint32_t           pieceOffset,
-                                tr_file_index_t *  fileIndex,
-                                uint64_t *         fileOffset );
+void tr_ioFindFileLocation( const tr_torrent  * tor,
+                             tr_piece_index_t   pieceIndex,
+                             uint32_t           pieceOffset,
+                             tr_file_index_t  * fileIndex,
+                             uint64_t         * fileOffset );
 
 
 /* @} */
