@@ -1,32 +1,32 @@
 /*
- * This file Copyright (C) 2009-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: metainfo.c 11398 2010-11-11 15:31:11Z charles $
+ * $Id: metainfo.c 12848 2011-09-06 16:45:48Z jordan $
  */
 
 #include <assert.h>
 #include <errno.h>
-#include <stdio.h>
-#include <string.h>
+#include <stdio.h> /* fopen(), fwrite(), fclose() */
+#include <string.h> /* strlen() */
 
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h> /* unlink, stat */
 
-#include <event.h> /* struct evbuffer */
+#include <event2/buffer.h>
 
 #include "transmission.h"
 #include "session.h"
 #include "bencode.h"
 #include "crypto.h" /* tr_sha1 */
 #include "metainfo.h"
-#include "platform.h"
+#include "platform.h" /* tr_getTorrentDir() */
 #include "utils.h"
 
 /***
@@ -36,22 +36,20 @@
 char*
 tr_metainfoGetBasename( const tr_info * inf )
 {
-    char *ret, *pch, *name;
+    size_t i;
+    const size_t name_len = strlen( inf->name );
+    char * ret = tr_strdup_printf( "%s.%16.16s", inf->name, inf->hashString );
 
-    name = tr_strdup( inf->name );
-    for( pch=name; pch && *pch; ++pch )
-        if( *pch == '/' )
-            *pch = '_';
+    for( i=0; i<name_len; ++i )
+        if( ret[i] == '/' )
+            ret[i] = '_';
 
-    ret = tr_strdup_printf( "%s.%16.16s", name, inf->hashString );
 
-    tr_free( name );
     return ret;
 }
 
 static char*
-getTorrentFilename( const tr_session * session,
-                    const tr_info *   inf )
+getTorrentFilename( const tr_session * session, const tr_info * inf )
 {
     char * base = tr_metainfoGetBasename( inf );
     char * filename = tr_strdup_printf( "%s" TR_PATH_DELIMITER_STR "%s.torrent",
@@ -139,24 +137,24 @@ tr_metainfoMigrate( tr_session * session,
 ****
 ***/
 
-static tr_bool
+static bool
 path_is_suspicious( const char * path )
 {
     return ( path == NULL )
         || ( strstr( path, "../" ) != NULL );
 }
 
-static tr_bool
-getfile( char ** setme, const char * root, tr_benc * path )
+static bool
+getfile( char ** setme, const char * root, tr_benc * path, struct evbuffer * buf )
 {
-    tr_bool success = FALSE;
+    bool success = false;
 
     if( tr_bencIsList( path ) )
     {
         int i;
         const int n = tr_bencListSize( path );
-        struct evbuffer * buf = evbuffer_new( );
 
+        evbuffer_drain( buf, evbuffer_get_length( buf ) );
         evbuffer_add( buf, root, strlen( root ) );
         for( i = 0; i < n; ++i )
         {
@@ -168,17 +166,16 @@ getfile( char ** setme, const char * root, tr_benc * path )
             }
         }
 
-        *setme = tr_utf8clean( (char*)EVBUFFER_DATA( buf ), EVBUFFER_LENGTH( buf ) );
+        *setme = tr_utf8clean( (char*)evbuffer_pullup( buf, -1 ), evbuffer_get_length( buf ) );
         /* fprintf( stderr, "[%s]\n", *setme ); */
-        evbuffer_free( buf );
-        success = TRUE;
+        success = true;
     }
 
     if( ( *setme != NULL ) && path_is_suspicious( *setme ) )
     {
         tr_free( *setme );
         *setme = NULL;
-        success = FALSE;
+        success = false;
     }
 
     return success;
@@ -194,6 +191,7 @@ parseFiles( tr_info * inf, tr_benc * files, const tr_benc * length )
     if( tr_bencIsList( files ) ) /* multi-file mode */
     {
         tr_file_index_t i;
+        struct evbuffer * buf = evbuffer_new( );
 
         inf->isMultifile = 1;
         inf->fileCount   = tr_bencListSize( files );
@@ -212,7 +210,7 @@ parseFiles( tr_info * inf, tr_benc * files, const tr_benc * length )
                 if( !tr_bencDictFindList( file, "path", &path ) )
                     return "path";
 
-            if( !getfile( &inf->files[i].name, inf->name, path ) )
+            if( !getfile( &inf->files[i].name, inf->name, path, buf ) )
                 return "path";
 
             if( !tr_bencDictFindInt( file, "length", &len ) )
@@ -221,6 +219,8 @@ parseFiles( tr_info * inf, tr_benc * files, const tr_benc * length )
             inf->files[i].length = len;
             inf->totalSize      += len;
         }
+
+        evbuffer_free( buf );
     }
     else if( tr_bencGetInt( length, &len ) ) /* single-file mode */
     {
@@ -253,7 +253,7 @@ tr_convertAnnounceToScrape( const char * announce )
      * If the text immediately following that '/' isn't 'announce'
      * it will be taken as a sign that that tracker doesn't support
      * the scrape convention. If it does, substitute 'scrape' for
-     * 'announce' to find the scrape page.  */
+     * 'announce' to find the scrape page. */
     if( ( ( s = strrchr( announce, '/' ) ) ) && !strncmp( ++s, "announce", 8 ) )
     {
         const char * prefix = announce;
@@ -297,24 +297,24 @@ getannounce( tr_info * inf, tr_benc * meta )
         {
             tr_benc * tier = tr_bencListChild( tiers, i );
             const int tierSize = tr_bencListSize( tier );
-            tr_bool anyAdded = FALSE;
+            bool anyAdded = false;
             for( j = 0; j < tierSize; ++j )
             {
                 if( tr_bencGetStr( tr_bencListChild( tier, j ), &str ) )
                 {
                     char * url = tr_strstrip( tr_strdup( str ) );
-                    if( tr_urlIsValidTracker( url ) )
-                    {
+                    if( !tr_urlIsValidTracker( url ) )
+                        tr_free( url );
+                    else {
                         tr_tracker_info * t = trackers + trackerCount;
                         t->tier = validTiers;
-                        t->announce = tr_strdup( url );
+                        t->announce = url;
                         t->scrape = tr_convertAnnounceToScrape( url );
                         t->id = trackerCount;
 
-                        anyAdded = TRUE;
+                        anyAdded = true;
                         ++trackerCount;
                     }
-                    tr_free( url );
                 }
             }
 
@@ -335,17 +335,17 @@ getannounce( tr_info * inf, tr_benc * meta )
       && tr_bencDictFindStr( meta, "announce", &str ) )
     {
         char * url = tr_strstrip( tr_strdup( str ) );
-        if( tr_urlIsValidTracker( url ) )
-        {
+        if( !tr_urlIsValidTracker( url ) )
+            tr_free( url );
+        else {
             trackers = tr_new0( tr_tracker_info, 1 );
             trackers[trackerCount].tier = 0;
-            trackers[trackerCount].announce = tr_strdup( url );
+            trackers[trackerCount].announce = url;
             trackers[trackerCount].scrape = tr_convertAnnounceToScrape( url );
             trackers[trackerCount].id = 0;
             trackerCount++;
             /*fprintf( stderr, "single announce: [%s]\n", url );*/
         }
-        tr_free( url );
     }
 
     inf->trackers = trackers;
@@ -354,6 +354,34 @@ getannounce( tr_info * inf, tr_benc * meta )
     return NULL;
 }
 
+/**
+ * @brief Ensure that the URLs for multfile torrents end in a slash.
+ *
+ * See http://bittorrent.org/beps/bep_0019.html#metadata-extension
+ * for background on how the trailing slash is used for "url-list"
+ * fields.
+ *
+ * This function is to workaround some .torrent generators, such as
+ * mktorrent and very old versions of utorrent, that don't add the
+ * trailing slash for multifile torrents if omitted by the end user.
+ */
+static char*
+fix_webseed_url( const tr_info * inf, const char * url )
+{
+    char * ret = NULL;
+    const size_t len = strlen( url );
+
+    if( tr_urlIsValid( url, len ) )
+    {
+        if( ( inf->fileCount > 1 ) && ( len > 0 ) && ( url[len-1] != '/' ) )
+            ret = tr_strdup_printf( "%*.*s/", (int)len, (int)len, url );
+        else
+            ret = tr_strndup( url, len );
+    }
+
+    return ret;
+}
+
 static void
 geturllist( tr_info * inf,
             tr_benc * meta )
@@ -370,47 +398,33 @@ geturllist( tr_info * inf,
         inf->webseeds = tr_new0( char*, n );
 
         for( i = 0; i < n; ++i )
+        {
             if( tr_bencGetStr( tr_bencListChild( urls, i ), &url ) )
-                inf->webseeds[inf->webseedCount++] = tr_strdup( url );
+            {
+                char * fixed_url = fix_webseed_url( inf, url );
+
+                if( fixed_url != NULL )
+                    inf->webseeds[inf->webseedCount++] = fixed_url;
+            }
+        }
     }
     else if( tr_bencDictFindStr( meta, "url-list", &url ) ) /* handle single items in webseeds */
     {
-        inf->webseedCount = 1;
-        inf->webseeds = tr_new0( char*, 1 );
-        inf->webseeds[0] = tr_strdup( url );
+        char * fixed_url = fix_webseed_url( inf, url );
+
+        if( fixed_url != NULL )
+        {
+            inf->webseedCount = 1;
+            inf->webseeds = tr_new0( char*, 1 );
+            inf->webseeds[0] = fixed_url;
+        }
     }
 }
 
-static int
-is_rfc2396_alnum( char ch )
-{
-    return ( '0' <= ch && ch <= '9' )
-        || ( 'A' <= ch && ch <= 'Z' )
-        || ( 'a' <= ch && ch <= 'z' )
-        || ch == '.'
-        || ch == '-'
-        || ch == '_'
-        || ch == '~';
-}
-
-static void
-escape( char * out, const uint8_t * in, size_t in_len ) /* rfc2396 */
-{
-    const uint8_t *end = in + in_len;
-
-    while( in != end )
-        if( is_rfc2396_alnum( *in ) )
-            *out++ = (char) *in++;
-        else
-            out += tr_snprintf( out, 4, "%%%02x", (unsigned int)*in++ );
-
-    *out = '\0';
-}
-
 static const char*
 tr_metainfoParseImpl( const tr_session  * session,
                       tr_info           * inf,
-                      tr_bool           * hasInfoDict,
+                      bool              * hasInfoDict,
                       int               * infoDictLength,
                       const tr_benc     * meta_in )
 {
@@ -421,8 +435,8 @@ tr_metainfoParseImpl( const tr_session  * session,
     tr_benc *       d;
     tr_benc *       infoDict = NULL;
     tr_benc *       meta = (tr_benc *) meta_in;
-    tr_bool         b;
-    tr_bool         isMagnet = FALSE;
+    bool            b;
+    bool            isMagnet = false;
 
     /* info_hash: urlencoded 20-byte SHA1 hash of the value of the info key
      * from the Metainfo file. Note that the value will be a bencoded
@@ -435,7 +449,7 @@ tr_metainfoParseImpl( const tr_session  * session,
         /* no info dictionary... is this a magnet link? */
         if( tr_bencDictFindDict( meta, "magnet-info", &d ) )
         {
-            isMagnet = TRUE;
+            isMagnet = true;
 
             /* get the info-hash */
             if( !tr_bencDictFindRaw( d, "info_hash", &raw, &raw_len ) )
@@ -444,7 +458,6 @@ tr_metainfoParseImpl( const tr_session  * session,
                 return "info_hash";
             memcpy( inf->hash, raw, raw_len );
             tr_sha1_to_hex( inf->hashString, inf->hash );
-            escape( inf->hashEscaped, inf->hash, SHA_DIGEST_LENGTH );
 
             /* maybe get the display name */
             if( tr_bencDictFindStr( d, "display-name", &str ) ) {
@@ -466,7 +479,6 @@ tr_metainfoParseImpl( const tr_session  * session,
         char * bstr = tr_bencToStr( infoDict, TR_FMT_BENC, &len );
         tr_sha1( inf->hash, bstr, len, NULL );
         tr_sha1_to_hex( inf->hashString, inf->hash );
-        escape( inf->hashEscaped, inf->hash, SHA_DIGEST_LENGTH );
 
         if( infoDictLength != NULL )
             *infoDictLength = len;
@@ -556,11 +568,11 @@ tr_metainfoParseImpl( const tr_session  * session,
     return NULL;
 }
 
-tr_bool
+bool
 tr_metainfoParse( const tr_session * session,
                   const tr_benc    * meta_in,
                   tr_info          * inf,
-                  tr_bool          * hasInfoDict,
+                  bool             * hasInfoDict,
                   int              * infoDictLength )
 {
     const char * badTag = tr_metainfoParseImpl( session,
@@ -568,7 +580,7 @@ tr_metainfoParse( const tr_session * session,
                                                 hasInfoDict,
                                                 infoDictLength,
                                                 meta_in );
-    const tr_bool success = badTag == NULL;
+    const bool success = badTag == NULL;
 
     if( badTag )
     {
@@ -610,8 +622,7 @@ tr_metainfoFree( tr_info * inf )
 }
 
 void
-tr_metainfoRemoveSaved( const tr_session * session,
-                        const tr_info *   inf )
+tr_metainfoRemoveSaved( const tr_session * session, const tr_info * inf )
 {
     char * filename;
 
