@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2008-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: peer-common.h 10800 2010-06-19 14:33:10Z charles $
+ * $Id: peer-common.h 12204 2011-03-22 15:19:54Z jordan $
  */
 
 #ifndef __TRANSMISSION__
@@ -28,6 +28,8 @@
 
 #include "transmission.h"
 
+struct tr_bitfield;
+
 enum
 {
     /** when we're making requests from another peer,
@@ -39,19 +41,8 @@ enum
         most bittorrent clients will reject requests
         larger than this size. */
     MAX_BLOCK_SIZE = ( 1024 * 16 )
-    
 };
 
-typedef enum
-{
-    TR_ADDREQ_OK = 0,
-    TR_ADDREQ_FULL,
-    TR_ADDREQ_DUPLICATE,
-    TR_ADDREQ_MISSING,
-    TR_ADDREQ_CLIENT_CHOKED
-}
-tr_addreq_t;
-
 /**
 ***  Peer Publish / Subscribe
 **/
@@ -65,30 +56,40 @@ typedef enum
     TR_PEER_CLIENT_GOT_SUGGEST,
     TR_PEER_CLIENT_GOT_PORT,
     TR_PEER_CLIENT_GOT_REJ,
+    TR_PEER_CLIENT_GOT_BITFIELD,
+    TR_PEER_CLIENT_GOT_HAVE,
+    TR_PEER_CLIENT_GOT_HAVE_ALL,
+    TR_PEER_CLIENT_GOT_HAVE_NONE,
     TR_PEER_PEER_GOT_DATA,
-    TR_PEER_PEER_PROGRESS,
     TR_PEER_ERROR
 }
 PeerEventType;
 
 typedef struct
 {
-    PeerEventType    eventType;
-    uint32_t         pieceIndex;   /* for GOT_BLOCK, CANCEL, ALLOWED, SUGGEST */
-    uint32_t         offset;       /* for GOT_BLOCK */
-    uint32_t         length;       /* for GOT_BLOCK + GOT_DATA */
-    float            progress;     /* for PEER_PROGRESS */
-    int              err;          /* errno for GOT_ERROR */
-    tr_bool          wasPieceData; /* for GOT_DATA */
-    tr_port          port;         /* for GOT_PORT */
+    PeerEventType         eventType;
+
+    uint32_t              pieceIndex;   /* for GOT_BLOCK, GOT_HAVE, CANCEL, ALLOWED, SUGGEST */
+    struct tr_bitfield  * bitfield;     /* for GOT_BITFIELD */
+    uint32_t              offset;       /* for GOT_BLOCK */
+    uint32_t              length;       /* for GOT_BLOCK + GOT_DATA */
+    int                   err;          /* errno for GOT_ERROR */
+    bool                  wasPieceData; /* for GOT_DATA */
+    tr_port               port;         /* for GOT_PORT */
 }
 tr_peer_event;
 
+extern const tr_peer_event TR_PEER_EVENT_INIT;
+
 struct tr_peer;
 
-typedef void tr_peer_callback( struct tr_peer      * peer,
-                              const tr_peer_event  * event,
-                              void                 * client_data );
+typedef void tr_peer_callback( struct tr_peer       * peer,
+                               const tr_peer_event  * event,
+                               void                 * client_data );
+
+/** Update the tr_peer.progress field based on the 'have' bitset. */
+void tr_peerUpdateProgress( tr_torrent * tor, struct tr_peer * );
+
 
 #ifdef WIN32
  #define EMSGSIZE WSAEMSGSIZE
