@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2008-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: port-forwarding.h 9868 2010-01-04 21:00:47Z charles $
+ * $Id: port-forwarding.h 12224 2011-03-24 22:45:04Z jordan $
  */
 
 #ifndef __TRANSMISSION__
@@ -18,7 +18,6 @@
 #define SHARED_H 1
 
 #include "transmission.h"
-#include "net.h"
 
 /**
  * @addtogroup port_forwarding Port Forwarding
@@ -35,11 +34,11 @@ void       tr_sharedClose( tr_session * );
 
 void       tr_sharedPortChanged( tr_session * );
 
-void       tr_sharedTraversalEnable( tr_shared *, tr_bool isEnabled );
+void       tr_sharedTraversalEnable( tr_shared *, bool isEnabled );
 
 tr_port    tr_sharedGetPeerPort( const tr_shared * s );
 
-tr_bool    tr_sharedTraversalIsEnabled( const tr_shared * s );
+bool       tr_sharedTraversalIsEnabled( const tr_shared * s );
 
 int        tr_sharedTraversalStatus( const tr_shared * );
 
