@@ -1,17 +1,17 @@
 /*
- * This file Copyright (C) 2008-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: ptrarray.c 11398 2010-11-11 15:31:11Z charles $
+ * $Id: ptrarray.c 12606 2011-07-31 14:04:43Z jordan $
  */
 
 #include <assert.h>
-#include <stdlib.h>
+#include <stdlib.h> /* tr_renew() -> realloc() */
 #include <string.h> /* memmove */
 
 #include "ptrarray.h"
@@ -57,17 +57,6 @@ tr_ptrArrayPeek( tr_ptrArray * t,
     return t->items;
 }
 
-void*
-tr_ptrArrayNth( tr_ptrArray* t,
-                int          i )
-{
-    assert( t );
-    assert( i >= 0 );
-    assert( i < t->n_items );
-
-    return t->items[i];
-}
-
 int
 tr_ptrArrayInsert( tr_ptrArray * t,
                    void        * ptr,
@@ -113,8 +102,8 @@ tr_ptrArrayErase( tr_ptrArray * t,
     assert( end <= t->n_items );
 
     memmove( t->items + begin,
-            t->items + end,
-            sizeof( void* ) * ( t->n_items - end ) );
+             t->items + end,
+             sizeof( void* ) * ( t->n_items - end ) );
 
     t->n_items -= ( end - begin );
 }
@@ -124,67 +113,83 @@ tr_ptrArrayErase( tr_ptrArray * t,
 **/
 
 int
-tr_ptrArrayLowerBound( const tr_ptrArray *                t,
-                       const void *                       ptr,
-                       int                 compare( const void *,
-                                                    const void * ),
-                       tr_bool *                    exact_match )
+tr_ptrArrayLowerBound( const tr_ptrArray  * t,
+                       const void         * ptr,
+                       int                  compare( const void *, const void * ),
+                       bool               * exact_match )
 {
-    int len = t->n_items;
-    int first = 0;
+    int pos = -1;
+    bool match = false;
 
-    while( len > 0 )
+    if( t->n_items == 0 )
     {
-        int       half = len / 2;
-        int       middle = first + half;
-        const int c = compare( t->items[middle], ptr );
-        if( c < 0 )
+        pos = 0;
+    }
+    else
+    {
+        int first = 0;
+        int last = t->n_items - 1;
+
+        for( ;; )
         {
-            first = middle + 1;
-            len = len - half - 1;
-        }
-        else if( !c )
-        {
-            if( exact_match )
-                *exact_match = TRUE;
-            return middle;
-            break;
-        }
-        else
-        {
-            len = half;
+            const int half = (last - first) / 2;
+            const int c = compare( t->items[first + half], ptr );
+
+            if( c < 0 ) {
+                const int new_first = first + half + 1;
+                if( new_first > last ) {
+                    pos = new_first;
+                    break;
+                }
+                first = new_first;
+            }
+            else if( c > 0 ) {
+                const int new_last = first + half - 1;
+                if( new_last < first ) {
+                    pos = first;
+                    break;
+                }
+                last = new_last;
+            }
+            else {
+                match = true;
+                pos = first + half;
+                break;
+            }
         }
     }
 
     if( exact_match )
-        *exact_match = FALSE;
-
-    return first;
+        *exact_match = match;
+    return pos;
 }
 
-#ifdef NDEBUG
-#define assertSortedAndUnique(a,b)
-#else
 static void
-assertSortedAndUnique( const tr_ptrArray * t,
-                    int compare(const void*, const void*) )
+assertSortedAndUnique( const tr_ptrArray * t UNUSED,
+                       int compare(const void*, const void*) UNUSED )
 {
+#ifndef NDEBUG
     int i;
 
     for( i = 0; i < t->n_items - 2; ++i )
-        assert( compare( t->items[i], t->items[i + 1] ) <= 0 );
-}
+        assert( compare( t->items[i], t->items[i + 1] ) < 0 );
 #endif
+}
 
 int
 tr_ptrArrayInsertSorted( tr_ptrArray * t,
                          void *        ptr,
                          int           compare(const void*, const void*) )
 {
-    const int pos = tr_ptrArrayLowerBound( t, ptr, compare, NULL );
-    const int ret = tr_ptrArrayInsert( t, ptr, pos );
+    int pos;
+    int ret;
 
-    //assertSortedAndUnique( t, compare );
+    assertSortedAndUnique( t, compare );
+
+    pos = tr_ptrArrayLowerBound( t, ptr, compare, NULL );
+    ret = tr_ptrArrayInsert( t, ptr, pos );
+
+    assertSortedAndUnique( t, compare );
     return ret;
 }
 
@@ -193,9 +198,8 @@ tr_ptrArrayFindSorted( tr_ptrArray * t,
                        const void *  ptr,
                        int           compare(const void*, const void*) )
 {
-    tr_bool   match;
+    bool match = false;
     const int pos = tr_ptrArrayLowerBound( t, ptr, compare, &match );
-
     return match ? t->items[pos] : NULL;
 }
 
@@ -204,15 +208,18 @@ tr_ptrArrayRemoveSorted( tr_ptrArray * t,
                          const void  * ptr,
                          int           compare(const void*, const void*) )
 {
-    void *    ret = NULL;
-    tr_bool   match;
+    bool match;
+    void * ret = NULL;
     const int pos = tr_ptrArrayLowerBound( t, ptr, compare, &match );
+    assertSortedAndUnique( t, compare );
 
     if( match )
     {
         ret = t->items[pos];
+        assert( compare( ret, ptr ) == 0 );
         tr_ptrArrayErase( t, pos, pos + 1 );
     }
     assertSortedAndUnique( t, compare );
+    assert( ( ret == NULL ) || ( compare( ret, ptr ) == 0 ) );
     return ret;
 }
