@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2008-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: resume.c 11455 2010-12-04 14:53:53Z charles $
+ * $Id: resume.c 12921 2011-09-26 22:50:42Z jordan $
  */
 
 #include <unistd.h> /* unlink */
@@ -51,6 +51,8 @@
 #define KEY_SPEED_Bps              "speed-Bps"
 #define KEY_USE_GLOBAL_SPEED_LIMIT "use-global-speed-limit"
 #define KEY_USE_SPEED_LIMIT        "use-speed-limit"
+#define KEY_TIME_SEEDING           "seeding-time-seconds"
+#define KEY_TIME_DOWNLOADING       "downloading-time-seconds"
 #define KEY_SPEEDLIMIT_DOWN_SPEED  "down-speed"
 #define KEY_SPEEDLIMIT_DOWN_MODE   "down-mode"
 #define KEY_SPEEDLIMIT_UP_SPEED    "up-speed"
@@ -60,8 +62,10 @@
 #define KEY_IDLELIMIT_MINS         "idle-limit"
 #define KEY_IDLELIMIT_MODE         "idle-mode"
 
+#define KEY_PROGRESS_CHECKTIME "time-checked"
 #define KEY_PROGRESS_MTIMES    "mtimes"
 #define KEY_PROGRESS_BITFIELD  "bitfield"
+#define KEY_PROGRESS_BLOCKS    "blocks"
 #define KEY_PROGRESS_HAVE      "have"
 
 enum
@@ -89,25 +93,18 @@ savePeers( tr_benc * dict, const tr_torrent * tor )
     int count;
     tr_pex * pex;
 
-    count = tr_peerMgrGetPeers( (tr_torrent*) tor, &pex, TR_AF_INET, TR_PEERS_ALL, MAX_REMEMBERED_PEERS );
+    count = tr_peerMgrGetPeers( (tr_torrent*) tor, &pex, TR_AF_INET, TR_PEERS_INTERESTING, MAX_REMEMBERED_PEERS );
     if( count > 0 )
         tr_bencDictAddRaw( dict, KEY_PEERS, pex, sizeof( tr_pex ) * count );
     tr_free( pex );
 
-    count = tr_peerMgrGetPeers( (tr_torrent*) tor, &pex, TR_AF_INET6, TR_PEERS_ALL, MAX_REMEMBERED_PEERS );
+    count = tr_peerMgrGetPeers( (tr_torrent*) tor, &pex, TR_AF_INET6, TR_PEERS_INTERESTING, MAX_REMEMBERED_PEERS );
     if( count > 0 )
         tr_bencDictAddRaw( dict, KEY_PEERS6, pex, sizeof( tr_pex ) * count );
 
     tr_free( pex );
 }
 
-static tr_bool
-tr_isPex( const tr_pex * pex )
-{
-    return tr_isAddress( &pex->addr )
-        && ( pex->flags & 3 ) == pex->flags;
-}
-
 static int
 addPeers( tr_torrent * tor, const uint8_t * buf, int buflen )
 {
@@ -159,27 +156,24 @@ loadPeers( tr_benc * dict, tr_torrent * tor )
 ***/
 
 static void
-saveDND( tr_benc *          dict,
-         const tr_torrent * tor )
+saveDND( tr_benc * dict, const tr_torrent * tor )
 {
-    const tr_info *       inf = tr_torrentInfo( tor );
+    tr_benc * list;
+    tr_file_index_t i;
+    const tr_info * const inf = tr_torrentInfo( tor );
     const tr_file_index_t n = inf->fileCount;
-    tr_file_index_t       i;
-    tr_benc *             list;
 
     list = tr_bencDictAddList( dict, KEY_DND, n );
-    for( i = 0; i < n; ++i )
+    for( i=0; i<n; ++i )
         tr_bencListAddInt( list, inf->files[i].dnd ? 1 : 0 );
 }
 
 static uint64_t
-loadDND( tr_benc *    dict,
-         tr_torrent * tor )
+loadDND( tr_benc * dict, tr_torrent * tor )
 {
-    uint64_t              ret = 0;
-    tr_info *             inf = &tor->info;
-    const tr_file_index_t n = inf->fileCount;
-    tr_benc *             list = NULL;
+    uint64_t ret = 0;
+    tr_benc * list = NULL;
+    const tr_file_index_t n = tor->info.fileCount;
 
     if( tr_bencDictFindList( dict, KEY_DND, &list )
       && ( tr_bencListSize( list ) == n ) )
@@ -189,7 +183,7 @@ loadDND( tr_benc *    dict,
         tr_file_index_t * dnd = tr_new( tr_file_index_t, n );
         tr_file_index_t   i, dlCount = 0, dndCount = 0;
 
-        for( i = 0; i < n; ++i )
+        for( i=0; i<n; ++i )
         {
             if( tr_bencGetInt( tr_bencListChild( list, i ), &tmp ) && tmp )
                 dnd[dndCount++] = i;
@@ -199,13 +193,13 @@ loadDND( tr_benc *    dict,
 
         if( dndCount )
         {
-            tr_torrentInitFileDLs ( tor, dnd, dndCount, FALSE );
+            tr_torrentInitFileDLs ( tor, dnd, dndCount, false );
             tr_tordbg( tor, "Resume file found %d files listed as dnd",
                        dndCount );
         }
         if( dlCount )
         {
-            tr_torrentInitFileDLs ( tor, dl, dlCount, TRUE );
+            tr_torrentInitFileDLs ( tor, dl, dlCount, true );
             tr_tordbg( tor,
                        "Resume file found %d files marked for download",
                        dlCount );
@@ -219,7 +213,7 @@ loadDND( tr_benc *    dict,
     {
         tr_tordbg(
             tor,
-            "Couldn't load DND flags.  dnd list (%p) has %zu children; torrent has %d files",
+            "Couldn't load DND flags. DND list (%p) has %zu children; torrent has %d files",
             list, tr_bencListSize( list ), (int)n );
     }
 
@@ -233,10 +227,10 @@ loadDND( tr_benc *    dict,
 static void
 saveFilePriorities( tr_benc * dict, const tr_torrent * tor )
 {
-    const tr_info *       inf = tr_torrentInfo( tor );
+    tr_benc * list;
+    tr_file_index_t i;
+    const tr_info * const inf = tr_torrentInfo( tor );
     const tr_file_index_t n = inf->fileCount;
-    tr_file_index_t       i;
-    tr_benc *             list;
 
     list = tr_bencDictAddList( dict, KEY_FILE_PRIORITIES, n );
     for( i = 0; i < n; ++i )
@@ -246,10 +240,9 @@ saveFilePriorities( tr_benc * dict, const tr_torrent * tor )
 static uint64_t
 loadFilePriorities( tr_benc * dict, tr_torrent * tor )
 {
-    uint64_t              ret = 0;
-    tr_info *             inf = &tor->info;
-    const tr_file_index_t n = inf->fileCount;
-    tr_benc *             list;
+    tr_benc * list;
+    uint64_t ret = 0;
+    const tr_file_index_t n = tor->info.fileCount;
 
     if( tr_bencDictFindList( dict, KEY_FILE_PRIORITIES, &list )
       && ( tr_bencListSize( list ) == n ) )
@@ -305,7 +298,7 @@ static void
 loadSingleSpeedLimit( tr_benc * d, tr_direction dir, tr_torrent * tor )
 {
     int64_t i;
-    tr_bool boolVal;
+    bool boolVal;
 
     if( tr_bencDictFindInt( d, KEY_SPEED_Bps, &i ) )
         tr_torrentSetSpeedLimit_Bps( tor, dir, i );
@@ -328,8 +321,9 @@ enum old_speed_modes
 static uint64_t
 loadSpeedLimits( tr_benc * dict, tr_torrent * tor )
 {
-    uint64_t  ret = 0;
     tr_benc * d;
+    uint64_t ret = 0;
+
 
     if( tr_bencDictFindDict( dict, KEY_SPEEDLIMIT_UP, &d ) )
     {
@@ -366,11 +360,10 @@ loadSpeedLimits( tr_benc * dict, tr_torrent * tor )
 }
 
 static uint64_t
-loadRatioLimits( tr_benc *    dict,
-                 tr_torrent * tor )
+loadRatioLimits( tr_benc * dict, tr_torrent * tor )
 {
-    uint64_t  ret = 0;
     tr_benc * d;
+    uint64_t ret = 0;
 
     if( tr_bencDictFindDict( dict, KEY_RATIOLIMIT, &d ) )
     {
@@ -387,11 +380,10 @@ loadRatioLimits( tr_benc *    dict,
 }
 
 static uint64_t
-loadIdleLimits( tr_benc *    dict,
-                      tr_torrent * tor )
+loadIdleLimits( tr_benc * dict, tr_torrent * tor )
 {
-    uint64_t  ret = 0;
     tr_benc * d;
+    uint64_t ret = 0;
 
     if( tr_bencDictFindDict( dict, KEY_IDLELIMIT, &d ) )
     {
@@ -411,120 +403,215 @@ loadIdleLimits( tr_benc *    dict,
 ***/
 
 static void
-saveProgress( tr_benc *          dict,
-              const tr_torrent * tor )
+bitfieldToBenc( const tr_bitfield * b, tr_benc * benc )
 {
-    size_t              i, n;
-    time_t *            mtimes;
-    tr_benc *           p;
-    tr_benc *           m;
-    const tr_bitfield * bitfield;
+    if( tr_bitfieldHasAll( b ) )
+        tr_bencInitStr( benc, "all", 3 );
+    else if( tr_bitfieldHasNone( b ) )
+        tr_bencInitStr( benc, "none", 4 );
+    else {
+        size_t byte_count = 0;
+        uint8_t * raw = tr_bitfieldGetRaw( b, &byte_count );
+        tr_bencInitRaw( benc, raw, byte_count );
+        tr_free( raw );
+    }
+}
 
-    p = tr_bencDictAdd( dict, KEY_PROGRESS );
-    tr_bencInitDict( p, 2 );
 
-    /* add the mtimes */
-    mtimes = tr_torrentGetMTimes( tor, &n );
-    m = tr_bencDictAddList( p, KEY_PROGRESS_MTIMES, n );
-    for( i = 0; i < n; ++i )
+static void
+saveProgress( tr_benc * dict, tr_torrent * tor )
+{
+    tr_benc * l;
+    tr_benc * prog;
+    tr_file_index_t fi;
+    const tr_info * inf = tr_torrentInfo( tor );
+    const time_t now = tr_time( );
+
+    prog = tr_bencDictAddDict( dict, KEY_PROGRESS, 3 );
+
+    /* add the file/piece check timestamps... */
+    l = tr_bencDictAddList( prog, KEY_PROGRESS_CHECKTIME, inf->fileCount );
+    for( fi=0; fi<inf->fileCount; ++fi )
     {
-        if( !tr_torrentIsFileChecked( tor, i ) )
-            mtimes[i] = ~(time_t)0; /* force a recheck */
-        tr_bencListAddInt( m, mtimes[i] );
+        const tr_piece * p;
+        const tr_piece * pend;
+        time_t oldest_nonzero = now;
+        time_t newest = 0;
+        bool has_zero = false;
+        const time_t mtime = tr_torrentGetFileMTime( tor, fi );
+        const tr_file * f = &inf->files[fi];
+
+        /* get the oldest and newest nonzero timestamps for pieces in this file */
+        for( p=&inf->pieces[f->firstPiece], pend=&inf->pieces[f->lastPiece]; p!=pend; ++p )
+        {
+            if( !p->timeChecked )
+                has_zero = true;
+            else if( oldest_nonzero > p->timeChecked )
+                oldest_nonzero = p->timeChecked;
+            if( newest < p->timeChecked )
+                newest = p->timeChecked;
+        }
+
+        /* If some of a file's pieces have been checked more recently than
+           the file's mtime, and some lest recently, then that file will
+           have a list containing timestamps for each piece.
+
+           However, the most common use case is that the file doesn't change
+           after it's downloaded. To reduce overhead in the .resume file,
+           only a single timestamp is saved for the file if *all* or *none*
+           of the pieces were tested more recently than the file's mtime. */
+
+        if( !has_zero && ( mtime <= oldest_nonzero ) ) /* all checked */
+            tr_bencListAddInt( l, oldest_nonzero );
+        else if( newest < mtime ) /* none checked */
+            tr_bencListAddInt( l, newest );
+        else { /* some are checked, some aren't... so list piece by piece */
+            const int offset = oldest_nonzero - 1;
+            tr_benc * ll = tr_bencListAddList( l, 2 + f->lastPiece - f->firstPiece );
+            tr_bencListAddInt( ll, offset );
+            for( p=&inf->pieces[f->firstPiece], pend=&inf->pieces[f->lastPiece]+1; p!=pend; ++p )
+                tr_bencListAddInt( ll, p->timeChecked ? p->timeChecked - offset : 0 );
+        }
     }
 
     /* add the progress */
     if( tor->completeness == TR_SEED )
-        tr_bencDictAddStr( p, KEY_PROGRESS_HAVE, "all" );
-    bitfield = tr_cpBlockBitfield( &tor->completion );
-    tr_bencDictAddRaw( p, KEY_PROGRESS_BITFIELD,
-                       bitfield->bits, bitfield->byteCount );
+        tr_bencDictAddStr( prog, KEY_PROGRESS_HAVE, "all" );
 
-    /* cleanup */
-    tr_free( mtimes );
+    /* add the blocks bitfield */
+    bitfieldToBenc( &tor->completion.blockBitfield,
+                    tr_bencDictAdd( prog, KEY_PROGRESS_BLOCKS ) );
 }
 
 static uint64_t
-loadProgress( tr_benc *    dict,
-              tr_torrent * tor )
+loadProgress( tr_benc * dict, tr_torrent * tor )
 {
-    uint64_t  ret = 0;
-    tr_benc * p;
+    size_t i, n;
+    uint64_t ret = 0;
+    tr_benc * prog;
+    const tr_info * inf = tr_torrentInfo( tor );
 
-    if( tr_bencDictFindDict( dict, KEY_PROGRESS, &p ) )
+    for( i=0, n=inf->pieceCount; i<n; ++i )
+        inf->pieces[i].timeChecked = 0;
+
+    if( tr_bencDictFindDict( dict, KEY_PROGRESS, &prog ) )
     {
         const char * err;
         const char * str;
         const uint8_t * raw;
-        size_t          rawlen;
-        tr_benc *       m;
-        size_t          n;
-        time_t *        curMTimes = tr_torrentGetMTimes( tor, &n );
+        size_t rawlen;
+        tr_benc * l;
+        tr_benc * b;
+        struct tr_bitfield blocks = TR_BITFIELD_INIT;
 
-        if( tr_bencDictFindList( p, KEY_PROGRESS_MTIMES, &m )
-          && ( n == tor->info.fileCount )
-          && ( n == tr_bencListSize( m ) ) )
+        if( tr_bencDictFindList( prog, KEY_PROGRESS_CHECKTIME, &l ) )
         {
-            size_t i;
-            for( i = 0; i < n; ++i )
+            /* per-piece timestamps were added in 2.20.
+
+               If some of a file's pieces have been checked more recently than
+               the file's mtime, and some lest recently, then that file will
+               have a list containing timestamps for each piece.
+
+               However, the most common use case is that the file doesn't change
+               after it's downloaded. To reduce overhead in the .resume file,
+               only a single timestamp is saved for the file if *all* or *none*
+               of the pieces were tested more recently than the file's mtime. */
+
+            tr_file_index_t fi;
+
+            for( fi=0; fi<inf->fileCount; ++fi )
             {
-                int64_t tmp;
-                if( !tr_bencGetInt( tr_bencListChild( m, i ), &tmp ) )
+                tr_benc * b = tr_bencListChild( l, fi );
+                const tr_file * f = &inf->files[fi];
+                tr_piece * p = &inf->pieces[f->firstPiece];
+                const tr_piece * pend = &inf->pieces[f->lastPiece]+1;
+
+                if( tr_bencIsInt( b ) )
                 {
-                    tr_tordbg(
-                        tor,
-                        "File #%zu needs to be verified - couldn't find benc entry",
-                        i );
-                    tr_torrentSetFileChecked( tor, i, FALSE );
+                    int64_t t;
+                    tr_bencGetInt( b, &t );
+                    for( ; p!=pend; ++p )
+                        p->timeChecked = (time_t)t;
                 }
-                else
+                else if( tr_bencIsList( b ) )
                 {
-                    const time_t t = (time_t) tmp;
-                    if( t == curMTimes[i] )
-                        tr_torrentSetFileChecked( tor, i, TRUE );
-                    else
+                    int i = 0;
+                    int64_t offset = 0;
+                    const int pieces = f->lastPiece + 1 - f->firstPiece;
+
+                    tr_bencGetInt( tr_bencListChild( b, 0 ), &offset );
+
+                    for( i=0; i<pieces; ++i )
                     {
-                        tr_tordbg(
-                            tor,
-                            "File #%zu needs to be verified - times %lu and %lu don't match",
-                            i, t, curMTimes[i] );
-                        tr_torrentSetFileChecked( tor, i, FALSE );
+                        int64_t t = 0;
+                        tr_bencGetInt( tr_bencListChild( b, i+1 ), &t );
+                        inf->pieces[f->firstPiece+i].timeChecked = (time_t)(t ? t + offset : 0);
                     }
                 }
             }
         }
-        else
+        else if( tr_bencDictFindList( prog, KEY_PROGRESS_MTIMES, &l ) )
         {
-            tr_torrentUncheck( tor );
-            tr_tordbg(
-                tor, "Torrent needs to be verified - unable to find mtimes" );
+            tr_file_index_t fi;
+
+            /* Before 2.20, we stored the files' mtimes in the .resume file.
+               When loading the .resume file, a torrent's file would be flagged
+               as untested if its stored mtime didn't match its real mtime. */
+
+            for( fi=0; fi<inf->fileCount; ++fi )
+            {
+                int64_t t;
+
+                if( tr_bencGetInt( tr_bencListChild( l, fi ), &t ) )
+                {
+                    const tr_file * f = &inf->files[fi];
+                    tr_piece * p = &inf->pieces[f->firstPiece];
+                    const tr_piece * pend = &inf->pieces[f->lastPiece];
+                    const time_t mtime = tr_torrentGetFileMTime( tor, fi );
+                    const time_t timeChecked = mtime==t ? mtime : 0;
+
+                    for( ; p!=pend; ++p )
+                        p->timeChecked = timeChecked;
+                }
+            }
         }
 
         err = NULL;
-        if( tr_bencDictFindStr( p, KEY_PROGRESS_HAVE, &str ) )
+        tr_bitfieldConstruct( &blocks, tor->blockCount );
+
+        if(( b = tr_bencDictFind( prog, KEY_PROGRESS_BLOCKS )))
+        {
+            size_t buflen;
+            const uint8_t * buf;
+
+            if( !tr_bencGetRaw( b, &buf, &buflen ) )
+                err = "Invalid value for \"blocks\"";
+            else if( ( buflen == 3 ) && !memcmp( buf, "all", 3 ) )
+                tr_bitfieldSetHasAll( &blocks );
+            else if( ( buflen == 4 ) && !memcmp( buf, "none", 4 ) )
+                tr_bitfieldSetHasNone( &blocks );
+            else
+                tr_bitfieldSetRaw( &blocks, buf, buflen, true );
+        }
+        else if( tr_bencDictFindStr( prog, KEY_PROGRESS_HAVE, &str ) )
         {
             if( !strcmp( str, "all" ) )
-                tr_cpSetHaveAll( &tor->completion );
+                tr_bitfieldSetHasAll( &blocks );
             else
                 err = "Invalid value for HAVE";
         }
-        else if( tr_bencDictFindRaw( p, KEY_PROGRESS_BITFIELD, &raw, &rawlen ) )
+        else if( tr_bencDictFindRaw( prog, KEY_PROGRESS_BITFIELD, &raw, &rawlen ) )
         {
-            tr_bitfield tmp;
-            tmp.byteCount = rawlen;
-            tmp.bitCount = tmp.byteCount * 8;
-            tmp.bits = (uint8_t*) raw;
-            if( !tr_cpBlockBitfieldSet( &tor->completion, &tmp ) )
-                err = "Error loading bitfield";
-        }
-        else err = "Couldn't find 'have' or 'bitfield'";
-        if( err != NULL )
-        {
-            tr_torrentUncheck( tor );
-            tr_tordbg( tor, "Torrent needs to be verified - %s", err );
+            tr_bitfieldSetRaw( &blocks, raw, rawlen, true );
         }
+        else err = "Couldn't find 'pieces' or 'have' or 'bitfield'";
 
-        tr_free( curMTimes );
+        if( err != NULL )
+            tr_tordbg( tor, "Torrent needs to be verified - %s", err );
+        else
+            tr_cpBlockInit( &tor->completion, &blocks );
+
+        tr_bitfieldDestruct( &blocks );
         ret = TR_FR_PROGRESS;
     }
 
@@ -546,6 +633,8 @@ tr_torrentSaveResume( tr_torrent * tor )
         return;
 
     tr_bencInitDict( &top, 50 ); /* arbitrary "big enough" number */
+    tr_bencDictAddInt( &top, KEY_TIME_SEEDING, tor->secondsSeeding );
+    tr_bencDictAddInt( &top, KEY_TIME_DOWNLOADING, tor->secondsDownloading );
     tr_bencDictAddInt( &top, KEY_ACTIVITY_DATE, tor->activityDate );
     tr_bencDictAddInt( &top, KEY_ADDED_DATE, tor->addedDate );
     tr_bencDictAddInt( &top, KEY_CORRUPT, tor->corruptPrev + tor->corruptCur );
@@ -578,16 +667,15 @@ tr_torrentSaveResume( tr_torrent * tor )
 }
 
 static uint64_t
-loadFromFile( tr_torrent * tor,
-              uint64_t     fieldsToLoad )
+loadFromFile( tr_torrent * tor, uint64_t fieldsToLoad )
 {
     int64_t  i;
     const char * str;
-    uint64_t fieldsLoaded = 0;
     char * filename;
     tr_benc top;
-    tr_bool boolVal;
-    const tr_bool  wasDirty = tor->isDirty;
+    bool boolVal;
+    uint64_t fieldsLoaded = 0;
+    const bool wasDirty = tor->isDirty;
 
     assert( tr_isTorrent( tor ) );
 
@@ -677,6 +765,20 @@ loadFromFile( tr_torrent * tor,
         fieldsLoaded |= TR_FR_ACTIVITY_DATE;
     }
 
+    if( ( fieldsToLoad & TR_FR_TIME_SEEDING )
+      && tr_bencDictFindInt( &top, KEY_TIME_SEEDING, &i ) )
+    {
+        tor->secondsSeeding = i;
+        fieldsLoaded |= TR_FR_TIME_SEEDING;
+    }
+
+    if( ( fieldsToLoad & TR_FR_TIME_DOWNLOADING )
+      && tr_bencDictFindInt( &top, KEY_TIME_DOWNLOADING, &i ) )
+    {
+        tor->secondsDownloading = i;
+        fieldsLoaded |= TR_FR_TIME_DOWNLOADING;
+    }
+
     if( ( fieldsToLoad & TR_FR_BANDWIDTH_PRIORITY )
       && tr_bencDictFindInt( &top, KEY_BANDWIDTH_PRIORITY, &i )
       && tr_isPriority( i ) )
@@ -717,16 +819,13 @@ loadFromFile( tr_torrent * tor,
 }
 
 static uint64_t
-setFromCtor( tr_torrent *    tor,
-             uint64_t        fields,
-             const tr_ctor * ctor,
-             int             mode )
+setFromCtor( tr_torrent * tor, uint64_t fields, const tr_ctor * ctor, int mode )
 {
     uint64_t ret = 0;
 
     if( fields & TR_FR_RUN )
     {
-        uint8_t isPaused;
+        bool isPaused;
         if( !tr_ctorGetPaused( ctor, mode, &isPaused ) )
         {
             tor->isRunning = !isPaused;
@@ -753,17 +852,13 @@ setFromCtor( tr_torrent *    tor,
 }
 
 static uint64_t
-useManditoryFields( tr_torrent *    tor,
-                    uint64_t        fields,
-                    const tr_ctor * ctor )
+useManditoryFields( tr_torrent * tor, uint64_t fields, const tr_ctor * ctor )
 {
     return setFromCtor( tor, fields, ctor, TR_FORCE );
 }
 
 static uint64_t
-useFallbackFields( tr_torrent *    tor,
-                   uint64_t        fields,
-                   const tr_ctor * ctor )
+useFallbackFields( tr_torrent * tor, uint64_t fields, const tr_ctor * ctor )
 {
     return setFromCtor( tor, fields, ctor, TR_FALLBACK );
 }
@@ -793,4 +888,3 @@ tr_torrentRemoveResume( const tr_torrent * tor )
     unlink( filename );
     tr_free( filename );
 }
-
