@@ -1,13 +1,13 @@
 /*
- * This file Copyright (C) 2009-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: torrent.h 11204 2010-09-12 18:58:49Z livings124 $
+ * $Id: torrent.h 12617 2011-08-03 23:40:51Z jordan $
  */
 
 #ifndef __TRANSMISSION__
@@ -17,11 +17,11 @@
 #ifndef TR_TORRENT_H
 #define TR_TORRENT_H 1
 
+#include "bandwidth.h" /* tr_bandwidth */
 #include "completion.h" /* tr_completion */
 #include "session.h" /* tr_sessionLock(), tr_sessionUnlock() */
 #include "utils.h" /* TR_GNUC_PRINTF */
 
-struct tr_bandwidth;
 struct tr_torrent_tiers;
 struct tr_magnet_info;
 
@@ -30,7 +30,7 @@ struct tr_magnet_info;
 **/
 
 void        tr_ctorSetSave( tr_ctor * ctor,
-                            tr_bool   saveMetadataInOurTorrentsDir );
+                            bool      saveMetadataInOurTorrentsDir );
 
 int         tr_ctorGetSave( const tr_ctor * ctor );
 
@@ -45,14 +45,14 @@ void        tr_ctorInitTorrentWanted( const tr_ctor * ctor, tr_torrent * tor );
 /* just like tr_torrentSetFileDLs but doesn't trigger a fastresume save */
 void        tr_torrentInitFileDLs( tr_torrent              * tor,
                                    const tr_file_index_t   * files,
-                                   tr_file_index_t          fileCount,
-                                   tr_bool                  do_download );
+                                   tr_file_index_t           fileCount,
+                                   bool                      do_download );
 
 void        tr_torrentRecheckCompleteness( tr_torrent * );
 
 void        tr_torrentSetHasPiece( tr_torrent *     tor,
                                    tr_piece_index_t pieceIndex,
-                                   tr_bool          has );
+                                   bool             has );
 
 void        tr_torrentChangeMyPort( tr_torrent * session );
 
@@ -62,7 +62,7 @@ tr_torrent* tr_torrentFindFromHashString( tr_session * session,
 tr_torrent* tr_torrentFindFromObfuscatedHash( tr_session    * session,
                                               const uint8_t * hash );
 
-tr_bool     tr_torrentIsPieceTransferAllowed( const tr_torrent * torrent,
+bool        tr_torrentIsPieceTransferAllowed( const tr_torrent * torrent,
                                               tr_direction       direction );
 
 
@@ -72,7 +72,7 @@ tr_block_index_t _tr_block( const tr_torrent * tor,
                             tr_piece_index_t   index,
                             uint32_t           offset );
 
-tr_bool          tr_torrentReqIsValid( const tr_torrent * tor,
+bool             tr_torrentReqIsValid( const tr_torrent * tor,
                                        tr_piece_index_t   index,
                                        uint32_t           offset,
                                        uint32_t           length );
@@ -82,30 +82,30 @@ uint64_t         tr_pieceOffset( const tr_torrent * tor,
                                  uint32_t           offset,
                                  uint32_t           length );
 
+void             tr_torrentGetBlockLocation( const tr_torrent * tor,
+                                             tr_block_index_t   block,
+                                             tr_piece_index_t * piece,
+                                             uint32_t         * offset,
+                                             uint32_t         * length );
+
+void             tr_torGetFileBlockRange( const tr_torrent        * tor,
+                                          const tr_file_index_t     file,
+                                          tr_block_index_t        * first,
+                                          tr_block_index_t        * last );
+
+void             tr_torGetPieceBlockRange( const tr_torrent        * tor,
+                                           const tr_piece_index_t    piece,
+                                           tr_block_index_t        * first,
+                                           tr_block_index_t        * last );
+
 void             tr_torrentInitFilePriority( tr_torrent       * tor,
                                              tr_file_index_t    fileIndex,
                                              tr_priority_t      priority );
 
-int              tr_torrentCountUncheckedPieces( const tr_torrent * );
-
-tr_bool          tr_torrentIsFileChecked( const tr_torrent  * tor,
-                                          tr_file_index_t     file );
-
 void             tr_torrentSetPieceChecked( tr_torrent       * tor,
-                                            tr_piece_index_t   piece,
-                                            tr_bool            isChecked );
+                                            tr_piece_index_t   piece );
 
-void             tr_torrentSetFileChecked( tr_torrent       * tor,
-                                           tr_file_index_t    file,
-                                           tr_bool            isChecked );
-
-void             tr_torrentUncheck( tr_torrent * tor );
-
-time_t*          tr_torrentGetMTimes( const tr_torrent  * tor,
-                                      size_t            * setmeCount );
-
-tr_torrent*      tr_torrentNext( tr_session  * session,
-                                 tr_torrent  * current );
+void             tr_torrentSetChecked( tr_torrent * tor, time_t when );
 
 void             tr_torrentCheckSeedLimit( tr_torrent * tor );
 
@@ -157,7 +157,7 @@ struct tr_torrent
      * peer_id that was registered by the peer. The peer_id from the tracker
      * and in the handshake are expected to match.
      */
-    uint8_t * peer_id;
+    uint8_t peer_id[PEER_ID_LEN+1];
 
     /* Where the files will be when it's complete */
     char * downloadDir;
@@ -190,16 +190,15 @@ struct tr_torrent
 
     struct tr_completion       completion;
 
-    struct tr_bitfield         checkedPieces;
     tr_completeness            completeness;
 
     struct tr_torrent_tiers  * tiers;
 
     time_t                     dhtAnnounceAt;
     time_t                     dhtAnnounce6At;
-    tr_bool                    dhtAnnounceInProgress;
-    tr_bool                    dhtAnnounce6InProgress;
-    
+    bool                       dhtAnnounceInProgress;
+    bool                       dhtAnnounce6InProgress;
+
     time_t                     lpdAnnounceAt;
 
     uint64_t                   downloadedCur;
@@ -210,9 +209,9 @@ struct tr_torrent
     uint64_t                   corruptPrev;
 
     uint64_t                   etaDLSpeedCalculatedAt;
-    double                     etaDLSpeed_KBps;
+    float                      etaDLSpeed_KBps;
     uint64_t                   etaULSpeedCalculatedAt;
-    double                     etaULSpeed_KBps;
+    float                      etaULSpeed_KBps;
 
     time_t                     addedDate;
     time_t                     activityDate;
@@ -220,6 +219,11 @@ struct tr_torrent
     time_t                     startDate;
     time_t                     anyDate;
 
+    int                        secondsDownloading;
+    int                        secondsSeeding;
+
+    int                        queuePosition;
+
     tr_torrent_metadata_func  * metadata_func;
     void                      * metadata_func_user_data;
 
@@ -232,13 +236,17 @@ struct tr_torrent
     tr_torrent_idle_limit_hit_func  * idle_limit_hit_func;
     void                            * idle_limit_hit_func_user_data;
 
-    tr_bool                    isRunning;
-    tr_bool                    isStopping;
-    tr_bool                    isDeleting;
-    tr_bool                    startAfterVerify;
-    tr_bool                    isDirty;
+    void * queue_started_user_data;
+    void ( * queue_started_callback )( tr_torrent *, void * queue_started_user_data );
 
-    tr_bool                    infoDictOffsetIsCached;
+    bool                       isRunning;
+    bool                       isStopping;
+    bool                       isDeleting;
+    bool                       startAfterVerify;
+    bool                       isDirty;
+    bool                       isQueued;
+
+    bool                       infoDictOffsetIsCached;
 
     uint16_t                   maxConnectedPeers;
 
@@ -251,25 +259,22 @@ struct tr_torrent
 
     int                        uniqueId;
 
-    struct tr_bandwidth      * bandwidth;
+    struct tr_bandwidth        bandwidth;
 
     struct tr_torrent_peers  * torrentPeers;
 
-    double                     desiredRatio;
+    float                      desiredRatio;
     tr_ratiolimit              ratioLimitMode;
 
     uint16_t                   idleLimitMinutes;
     tr_idlelimit               idleLimitMode;
-    tr_bool                    finishedSeedingByIdle;
-
-    uint64_t                   preVerifyTotal;
+    bool                       finishedSeedingByIdle;
 };
 
-/* get the index of this piece's first block */
-static inline tr_block_index_t
-tr_torPieceFirstBlock( const tr_torrent * tor, const tr_piece_index_t piece )
+static inline tr_torrent*
+tr_torrentNext( tr_session * session, tr_torrent * current )
 {
-    return piece * tor->blockCountInPiece;
+    return current ? current->next : session->torrentList;
 }
 
 /* what piece index is this block in? */
@@ -279,21 +284,11 @@ tr_torBlockPiece( const tr_torrent * tor, const tr_block_index_t block )
     return block / tor->blockCountInPiece;
 }
 
-/* how many blocks are in this piece? */
-static inline uint16_t
-tr_torPieceCountBlocks( const tr_torrent * tor, const tr_piece_index_t piece )
-{
-    if( piece + 1 == tor->info.pieceCount )
-        return tor->blockCountInLastPiece;
-    else
-        return tor->blockCountInPiece;
-}
-
 /* how many bytes are in this piece? */
 static inline uint32_t
 tr_torPieceCountBytes( const tr_torrent * tor, const tr_piece_index_t piece )
 {
-    return piece == tor->info.pieceCount - 1 ? tor->lastPieceSize
+    return piece + 1 == tor->info.pieceCount ? tor->lastPieceSize
                                              : tor->info.pieceSize;
 }
 
@@ -301,7 +296,7 @@ tr_torPieceCountBytes( const tr_torrent * tor, const tr_piece_index_t piece )
 static inline uint32_t
 tr_torBlockCountBytes( const tr_torrent * tor, const tr_block_index_t block )
 {
-    return block == tor->blockCount - 1 ? tor->lastBlockSize
+    return block + 1 == tor->blockCount ? tor->lastBlockSize
                                         : tor->blockSize;
 }
 
@@ -310,55 +305,54 @@ static inline void tr_torrentLock( const tr_torrent * tor )
     tr_sessionLock( tor->session );
 }
 
+static inline bool tr_torrentIsLocked( const tr_torrent * tor )
+{
+    return tr_sessionIsLocked( tor->session );
+}
+
 static inline void tr_torrentUnlock( const tr_torrent * tor )
 {
     tr_sessionUnlock( tor->session );
 }
 
-static inline tr_bool
+static inline bool
 tr_torrentExists( const tr_session * session, const uint8_t *   torrentHash )
 {
     return tr_torrentFindFromHash( (tr_session*)session, torrentHash ) != NULL;
 }
 
-static inline tr_bool
+static inline bool
 tr_torrentIsSeed( const tr_torrent * tor )
 {
     return tor->completeness != TR_LEECH;
 }
 
-static inline tr_bool tr_torrentIsPrivate( const tr_torrent * tor )
+static inline bool tr_torrentIsPrivate( const tr_torrent * tor )
 {
     return ( tor != NULL ) && tor->info.isPrivate;
 }
 
-static inline tr_bool tr_torrentAllowsPex( const tr_torrent * tor )
+static inline bool tr_torrentAllowsPex( const tr_torrent * tor )
 {
     return ( tor != NULL )
         && ( tor->session->isPexEnabled )
         && ( !tr_torrentIsPrivate( tor ) );
 }
 
-static inline tr_bool tr_torrentAllowsDHT( const tr_torrent * tor )
+static inline bool tr_torrentAllowsDHT( const tr_torrent * tor )
 {
     return ( tor != NULL )
         && ( tr_sessionAllowsDHT( tor->session ) )
         && ( !tr_torrentIsPrivate( tor ) );
 }
 
-static inline tr_bool tr_torrentAllowsLPD( const tr_torrent * tor )
+static inline bool tr_torrentAllowsLPD( const tr_torrent * tor )
 {
     return ( tor != NULL )
         && ( tr_sessionAllowsLPD( tor->session ) )
         && ( !tr_torrentIsPrivate( tor ) );
 }
 
-static inline tr_bool tr_torrentIsPieceChecked( const tr_torrent  * tor,
-                                                   tr_piece_index_t    i )
-{
-    return tr_bitfieldHasFast( &tor->checkedPieces, i );
-}
-
 /***
 ****
 ***/
@@ -368,7 +362,7 @@ enum
     TORRENT_MAGIC_NUMBER = 95549
 };
 
-static inline tr_bool tr_isTorrent( const tr_torrent * tor )
+static inline bool tr_isTorrent( const tr_torrent * tor )
 {
     return ( tor != NULL )
         && ( tor->magicNumber == TORRENT_MAGIC_NUMBER )
@@ -382,15 +376,7 @@ void tr_torrentSetDirty( tr_torrent * tor )
 {
     assert( tr_isTorrent( tor ) );
 
-    tor->isDirty = TRUE;
-}
-
-static inline
-const char * tr_torrentName( const tr_torrent * tor )
-{
-    assert( tr_isTorrent( tor ) );
-
-    return tor->info.name;
+    tor->isDirty = true;
 }
 
 uint32_t tr_getBlockSize( uint32_t pieceSize );
@@ -414,8 +400,8 @@ void tr_torrentFileCompleted( tr_torrent * tor, tr_file_index_t fileNo );
  * @param subpath on success, this pointer is assigned a newly-allocated
  *                string holding the second half of the filename.
  */
-tr_bool tr_torrentFindFile2( const tr_torrent *, tr_file_index_t fileNo,
-                             const char ** base, char ** subpath );
+bool tr_torrentFindFile2( const tr_torrent *, tr_file_index_t fileNo,
+                          const char ** base, char ** subpath, time_t * mtime );
 
 
 /* Returns a newly-allocated version of the tr_file.name string
@@ -431,5 +417,33 @@ void tr_torrentGotNewInfoDict( tr_torrent * tor );
 void tr_torrentSetSpeedLimit_Bps  ( tr_torrent *, tr_direction, int Bps );
 int tr_torrentGetSpeedLimit_Bps  ( const tr_torrent *, tr_direction );
 
+/**
+ * @return true if this piece needs to be tested
+ */
+bool tr_torrentPieceNeedsCheck( const tr_torrent * tor, tr_piece_index_t pieceIndex );
+
+/**
+ * @brief Test a piece against its info dict checksum
+ * @return true if the piece's passes the checksum test
+ */
+bool tr_torrentCheckPiece( tr_torrent * tor, tr_piece_index_t pieceIndex );
+
+time_t tr_torrentGetFileMTime( const tr_torrent * tor, tr_file_index_t i );
+
+uint64_t tr_torrentGetCurrentSizeOnDisk( const tr_torrent * tor );
+
+bool tr_torrentIsStalled( const tr_torrent * tor );
+
+static inline bool
+tr_torrentIsQueued( const tr_torrent * tor )
+{
+    return tor->isQueued;
+}
+
+static inline tr_direction
+tr_torrentGetQueueDirection( const tr_torrent * tor )
+{
+    return tr_torrentIsSeed( tor ) ? TR_UP : TR_DOWN;
+}
 
 #endif
