@@ -1,24 +1,23 @@
 /*
- * This file Copyright (C) 2007-2010 Mnemosyne LLC
+ * This file Copyright (C) Mnemosyne LLC
  *
- * This file is licensed by the GPL version 2.  Works owned by the
+ * This file is licensed by the GPL version 2. Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
  * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: trevent.c 10913 2010-06-30 21:24:36Z charles $
+ * $Id: trevent.c 12228 2011-03-25 01:41:57Z jordan $
  */
 
 #include <assert.h>
 #include <errno.h>
-#include <stdlib.h>
 #include <string.h>
-#include <stdio.h>
 
 #include <signal.h>
 
-#include <event.h>
+#include <event2/dns.h>
+#include <event2/event.h>
 
 #include "transmission.h"
 #include "net.h"
@@ -124,12 +123,10 @@ piperead( int s, char *buf, int len )
 #define pipewrite(a,b,c) write(a,b,c)
 #endif
 
-#include <unistd.h>
-
-#include <event.h>
+#include <unistd.h> /* read(), write(), pipe() */
 
 #include "transmission.h"
-#include "platform.h"
+#include "platform.h" /* tr_lockLock() */
 #include "trevent.h"
 #include "utils.h"
 
@@ -145,7 +142,7 @@ typedef struct tr_event_handle
     tr_session *  session;
     tr_thread *  thread;
     struct event_base * base;
-    struct event pipeEvent;
+    struct event * pipeEvent;
 }
 tr_event_handle;
 
@@ -200,7 +197,7 @@ readFromPipe( int    fd,
         case '\0': /* eof */
         {
             dbgmsg( "pipe eof reached... removing event listener" );
-            event_del( &eh->pipeEvent );
+            event_free( eh->pipeEvent );
             break;
         }
 
@@ -224,6 +221,7 @@ logFunc( int severity, const char * message )
 static void
 libeventThreadFunc( void * veh )
 {
+    struct event_base * base;
     tr_event_handle * eh = veh;
 
 #ifndef WIN32
@@ -231,21 +229,27 @@ libeventThreadFunc( void * veh )
     signal( SIGPIPE, SIG_IGN );
 #endif
 
-    eh->base = event_init( );
+    /* create the libevent bases */
+    base = event_base_new( );
+
+    /* set the struct's fields */
+    eh->base = base;
+    eh->session->event_base = base;
+    eh->session->evdns_base = evdns_base_new( base, true );
     eh->session->events = eh;
 
     /* listen to the pipe's read fd */
-    event_set( &eh->pipeEvent, eh->fds[0], EV_READ | EV_PERSIST, readFromPipe, veh );
-    event_add( &eh->pipeEvent, NULL );
+    eh->pipeEvent = event_new( base, eh->fds[0], EV_READ | EV_PERSIST, readFromPipe, veh );
+    event_add( eh->pipeEvent, NULL );
     event_set_log_callback( logFunc );
 
     /* loop until all the events are done */
     while( !eh->die )
-        event_dispatch( );
+        event_base_dispatch( base );
 
     /* shut down the thread */
     tr_lockFree( eh->lock );
-    event_base_free( eh->base );
+    event_base_free( base );
     eh->session->events = NULL;
     tr_free( eh );
     tr_dbg( "Closing libevent thread" );
@@ -274,7 +278,7 @@ tr_eventClose( tr_session * session )
 {
     assert( tr_isSession( session ) );
 
-    session->events->die = TRUE;
+    session->events->die = true;
     tr_deepLog( __FILE__, __LINE__, NULL, "closing trevent pipe" );
     tr_netCloseSocket( session->events->fds[1] );
 }
@@ -283,7 +287,7 @@ tr_eventClose( tr_session * session )
 ***
 **/
 
-tr_bool
+bool
 tr_amInEventThread( const tr_session * session )
 {
     assert( tr_isSession( session ) );
@@ -322,11 +326,3 @@ tr_runInEventThread( tr_session * session,
         tr_lockUnlock( lock );
     }
 }
-
-struct event_base *
-tr_eventGetBase( tr_session * session )
-{
-    assert( tr_isSession( session ) );
-
-    return session->events->base;
-}
