@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: Badger.m 10949 2010-07-06 03:40:34Z livings124 $
+ * $Id: Badger.m 13255 2012-03-13 03:49:30Z livings124 $
  *
- * Copyright (c) 2006-2010 Transmission authors and contributors
+ * Copyright (c) 2006-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -24,6 +24,8 @@
 
 #import "Badger.h"
 #import "BadgeView.h"
+#import "NSStringAdditions.h"
+#import "Torrent.h"
 
 @implementation Badger
 
@@ -33,14 +35,11 @@
     {
         fLib = lib;
         
-        fCompleted = 0;
-        
-        BadgeView * view = [[BadgeView alloc] initWithFrame: [[[NSApp dockTile] contentView] frame] lib: lib];
+        BadgeView * view = [[BadgeView alloc] initWithLib: lib];
         [[NSApp dockTile] setContentView: view];
         [view release];
         
-        //change that just impacts the dock badge
-        [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(updateBadge) name: @"DockBadgeChange" object: nil];
+        fHashes = [[NSMutableSet alloc] init];
     }
     
     return self;
@@ -48,34 +47,45 @@
 
 - (void) dealloc
 {
-    [[NSNotificationCenter defaultCenter] removeObserver: self];
-    
+    [fHashes release];
     [super dealloc];
 }
 
-- (void) updateBadge
+- (void) updateBadgeWithDownload: (CGFloat) downloadRate upload: (CGFloat) uploadRate
 {
-    const double downloadRate = [[NSUserDefaults standardUserDefaults] boolForKey: @"BadgeDownloadRate"]
-                                ? tr_sessionGetPieceSpeed_KBps(fLib, TR_DOWN) : 0.0;
-    const double uploadRate = [[NSUserDefaults standardUserDefaults] boolForKey: @"BadgeUploadRate"]
-                                ? tr_sessionGetPieceSpeed_KBps(fLib, TR_UP) : 0.0;
+    const CGFloat displayDlRate = [[NSUserDefaults standardUserDefaults] boolForKey: @"BadgeDownloadRate"]
+                                    ? downloadRate : 0.0;
+    const CGFloat displayUlRate = [[NSUserDefaults standardUserDefaults] boolForKey: @"BadgeUploadRate"]
+                                    ? uploadRate : 0.0;
     
     //only update if the badged values change
-    if ([(BadgeView *)[[NSApp dockTile] contentView] setRatesWithDownload: downloadRate upload: uploadRate])
+    if ([(BadgeView *)[[NSApp dockTile] contentView] setRatesWithDownload: displayDlRate upload: displayUlRate])
         [[NSApp dockTile] display];
 }
 
-- (void) incrementCompleted
+- (void) addCompletedTorrent: (Torrent *) torrent
 {
-    fCompleted++;
-    [[NSApp dockTile] setBadgeLabel: [NSString stringWithFormat: @"%d", fCompleted]];
+    [fHashes addObject: [torrent hashString]];
+    [[NSApp dockTile] setBadgeLabel: [NSString formattedUInteger: [fHashes count]]];
+}
+
+- (void) removeTorrent: (Torrent *) torrent
+{
+    if ([fHashes member: [torrent hashString]])
+    {
+        [fHashes removeObject: [torrent hashString]];
+        if ([fHashes count] > 0)
+            [[NSApp dockTile] setBadgeLabel: [NSString formattedUInteger: [fHashes count]]];
+        else
+            [[NSApp dockTile] setBadgeLabel: @""];
+    }
 }
 
 - (void) clearCompleted
 {
-    if (fCompleted != 0)
+    if ([fHashes count] > 0)
     {
-        fCompleted = 0;
+        [fHashes removeAllObjects];
         [[NSApp dockTile] setBadgeLabel: @""];
     }
 }
