@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: BonjourController.m 12596 2011-07-26 22:46:33Z livings124 $
+ * $Id: BonjourController.m 13253 2012-03-13 03:20:09Z livings124 $
  *
- * Copyright (c) 2008-2011 Transmission authors and contributors
+ * Copyright (c) 2008-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -29,8 +29,11 @@
 BonjourController * fDefaultController = nil;
 + (BonjourController *) defaultController
 {
-    if (!fDefaultController)
+    static dispatch_once_t onceToken;
+    dispatch_once(&onceToken, ^{
         fDefaultController = [[BonjourController alloc] init];
+    });
+    
     return fDefaultController;
 }
 
