@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: FilePriorityCell.m 9844 2010-01-01 21:12:04Z livings124 $
+ * $Id: FilePriorityCell.m 13251 2012-03-13 02:52:11Z livings124 $
  * 
- * Copyright (c) 2007-2010 Transmission authors and contributors
+ * Copyright (c) 2007-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -25,9 +25,11 @@
 #import "FilePriorityCell.h"
 #import "FileOutlineView.h"
 #import "FileListNode.h"
-#import "NSApplicationAdditions.h"
+#import "NSImageAdditions.h"
 #import "Torrent.h"
 
+#define IMAGE_OVERLAP 1.0
+
 @implementation FilePriorityCell
 
 - (id) init
@@ -53,6 +55,13 @@
     return self;
 }
 
+- (id) copyWithZone: (NSZone *) zone
+{
+    id value = [super copyWithZone: zone];
+    [value setRepresentedObject: [self representedObject]];
+    return value;
+}
+
 - (void) setSelected: (BOOL) flag forSegment: (NSInteger) segment
 {
     [super setSelected: flag forSegment: segment];
@@ -72,10 +81,11 @@
             break;
     }
     
-    FileOutlineView * controlView = (FileOutlineView *)[self controlView];
-    Torrent * torrent = [controlView torrent];
+    Torrent * torrent = [(FileListNode *)[self representedObject] torrent];
     [torrent setFilePriority: priority forIndexes: [(FileListNode *)[self representedObject] indexes]];
-    [controlView reloadData];
+    
+    FileOutlineView * controlView = (FileOutlineView *)[self controlView];
+    [controlView setNeedsDisplay: YES];
 }
 
 - (void) addTrackingAreasForView: (NSView *) controlView inRect: (NSRect) cellFrame withUserInfo: (NSDictionary *) userInfo
@@ -101,8 +111,8 @@
 
 - (void) drawWithFrame: (NSRect) cellFrame inView: (NSView *) controlView
 {
-    Torrent * torrent = [(FileOutlineView *)controlView torrent];
     FileListNode * node = [self representedObject];
+    Torrent * torrent = [node torrent];
     NSSet * priorities = [torrent filePrioritiesForIndexes: [node indexes]];
     
     const NSUInteger count = [priorities count];
@@ -116,40 +126,56 @@
     }
     else
     {
-        NSImage * image;
+        NSMutableArray * images = [NSMutableArray arrayWithCapacity: MAX(count, 1)];
+        CGFloat totalWidth;
+        
         if (count == 0)
-            image = [NSImage imageNamed: @"PriorityNone.png"];
-        else if (count > 1)
-            image = [NSImage imageNamed: @"PriorityMixed.png"];
+        {
+            //if ([self backgroundStyle] != NSBackgroundStyleDark)
+            {
+                NSImage * image = [[NSImage imageNamed: @"PriorityNormalTemplate.png"] imageWithColor: [NSColor lightGrayColor]];
+                [images addObject: image];
+                totalWidth = [image size].width;
+            }
+        }
         else
         {
-            switch ([[priorities anyObject] integerValue])
+            NSColor * priorityColor = [self backgroundStyle] == NSBackgroundStyleDark ? [NSColor whiteColor] : [NSColor darkGrayColor];
+            
+            totalWidth = 0.0;
+            if ([priorities containsObject: [NSNumber numberWithInteger: TR_PRI_LOW]])
             {
-                case TR_PRI_NORMAL:
-                    image = [NSImage imageNamed: @"PriorityNormal.png"];
-                    break;
-                case TR_PRI_LOW:
-                    image = [NSImage imageNamed: @"PriorityLow.png"];
-                    break;
-                case TR_PRI_HIGH:
-                    image = [NSImage imageNamed: @"PriorityHigh.png"];
-                    break;
+                NSImage * image = [[NSImage imageNamed: @"PriorityLowTemplate.png"] imageWithColor: priorityColor];
+                [images addObject: image];
+                totalWidth += [image size].width;
+            }
+            if ([priorities containsObject: [NSNumber numberWithInteger: TR_PRI_NORMAL]])
+            {
+                NSImage * image = [[NSImage imageNamed: @"PriorityNormalTemplate.png"] imageWithColor: priorityColor];
+                [images addObject: image];
+                totalWidth += [image size].width;
+            }
+            if ([priorities containsObject: [NSNumber numberWithInteger: TR_PRI_HIGH]])
+            {
+                NSImage * image = [[NSImage imageNamed: @"PriorityHighTemplate.png"] imageWithColor: priorityColor];
+                [images addObject: image];
+                totalWidth += [image size].width;
             }
         }
         
-        NSSize imageSize = [image size];
-        NSRect imageRect = NSMakeRect(NSMidX(cellFrame) - imageSize.width * 0.5, NSMidY(cellFrame) - imageSize.height * 0.5,
-                                        imageSize.width, imageSize.height);
+        if (count > 1)
+            totalWidth -= IMAGE_OVERLAP * (count-1);
         
-        if ([NSApp isOnSnowLeopardOrBetter])
-            [image drawInRect: imageRect fromRect: NSZeroRect operation: NSCompositeSourceOver fraction: 1.0
-                respectFlipped: YES hints: nil];
-        else
+        CGFloat currentWidth = floor(NSMidX(cellFrame) - totalWidth * 0.5);
+        
+        for (NSImage * image in images)
         {
-            image = [image copy];
-            [image setFlipped: YES];
-            [image drawInRect: imageRect fromRect: NSZeroRect operation: NSCompositeSourceOver fraction: 1.0];
-            [image release];
+            const NSSize imageSize = [image size];
+            const NSRect imageRect = NSMakeRect(currentWidth, floor(NSMidY(cellFrame) - imageSize.height * 0.5), imageSize.width, imageSize.height);
+            
+            [image drawInRect: imageRect fromRect: NSZeroRect operation: NSCompositeSourceOver fraction: 1.0 respectFlipped: YES hints: nil];
+            
+            currentWidth += imageSize.width - IMAGE_OVERLAP;
         }
     }
 }
