@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: MessageWindowController.m 12937 2011-10-03 01:38:55Z livings124 $
+ * $Id: MessageWindowController.m 13251 2012-03-13 02:52:11Z livings124 $
  *
- * Copyright (c) 2006-2011 Transmission authors and contributors
+ * Copyright (c) 2006-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -300,16 +300,8 @@
     NSString * messageString = [messageStrings componentsJoinedByString: @"\n"];
     
     NSPasteboard * pb = [NSPasteboard generalPasteboard];
-    if ([NSApp isOnSnowLeopardOrBetter])
-    {
-        [pb clearContents];
-        [pb writeObjects: [NSArray arrayWithObject: messageString]];
-    }
-    else
-    {
-        [pb declareTypes: [NSArray arrayWithObject: NSStringPboardType] owner: nil];
-        [pb setString: messageString forType: NSStringPboardType];
-    }
+    [pb clearContents];
+    [pb writeObjects: [NSArray arrayWithObject: messageString]];
 }
 
 - (BOOL) validateMenuItem: (NSMenuItem *) menuItem
@@ -386,48 +378,43 @@
 
 - (void) writeToFile: (id) sender
 {
-    //make the array sorted by date
-    NSSortDescriptor * descriptor = [[[NSSortDescriptor alloc] initWithKey: @"Index" ascending: YES] autorelease];
-    NSArray * descriptors = [[NSArray alloc] initWithObjects: descriptor, nil];
-    NSArray * sortedMessages = [[fDisplayedMessages sortedArrayUsingDescriptors: descriptors] retain];
-    [descriptors release];
-    
     NSSavePanel * panel = [NSSavePanel savePanel];
     [panel setAllowedFileTypes: [NSArray arrayWithObject: @"txt"]];
     [panel setCanSelectHiddenExtension: YES];
     
-    [panel beginSheetForDirectory: nil file: NSLocalizedString(@"untitled", "Save log panel -> default file name")
-            modalForWindow: [self window] modalDelegate: self
-            didEndSelector: @selector(writeToFileSheetClosed:returnCode:contextInfo:) contextInfo: sortedMessages];
-}
-
-- (void) writeToFileSheetClosed: (NSSavePanel *) panel returnCode: (NSInteger) code contextInfo: (NSArray *) messages
-{
-    if (code == NSOKButton)
-    {
-        //create the text to output
-        NSMutableArray * messageStrings = [NSMutableArray arrayWithCapacity: [messages count]];
-        for (NSDictionary * message in messages)
-            [messageStrings addObject: [self stringForMessage: message]];
+    [panel setNameFieldStringValue: NSLocalizedString(@"untitled", "Save log panel -> default file name")];
     
-        NSString * fileString = [messageStrings componentsJoinedByString: @"\n"];
-        
-        if (![fileString writeToFile: [panel filename] atomically: YES encoding: NSUTF8StringEncoding error: nil])
+    [panel beginSheetModalForWindow: [self window] completionHandler: ^(NSInteger result) {
+        if (result == NSFileHandlingPanelOKButton)
         {
-            NSAlert * alert = [[NSAlert alloc] init];
-            [alert addButtonWithTitle: NSLocalizedString(@"OK", "Save log alert panel -> button")];
-            [alert setMessageText: NSLocalizedString(@"Log Could Not Be Saved", "Save log alert panel -> title")];
-            [alert setInformativeText: [NSString stringWithFormat:
-                    NSLocalizedString(@"There was a problem creating the file \"%@\".",
-                    "Save log alert panel -> message"), [[panel filename] lastPathComponent]]];
-            [alert setAlertStyle: NSWarningAlertStyle];
+            //make the array sorted by date
+            NSSortDescriptor * descriptor = [NSSortDescriptor sortDescriptorWithKey: @"Index" ascending: YES];
+            NSArray * descriptors = [[NSArray alloc] initWithObjects: descriptor, nil];
+            NSArray * sortedMessages = [fDisplayedMessages sortedArrayUsingDescriptors: descriptors];
+            [descriptors release];
             
-            [alert runModal];
-            [alert release];
+            //create the text to output
+            NSMutableArray * messageStrings = [NSMutableArray arrayWithCapacity: [sortedMessages count]];
+            for (NSDictionary * message in sortedMessages)
+                [messageStrings addObject: [self stringForMessage: message]];
+            
+            NSString * fileString = [messageStrings componentsJoinedByString: @"\n"];
+            
+            if (![fileString writeToFile: [[panel URL] path] atomically: YES encoding: NSUTF8StringEncoding error: nil])
+            {
+                NSAlert * alert = [[NSAlert alloc] init];
+                [alert addButtonWithTitle: NSLocalizedString(@"OK", "Save log alert panel -> button")];
+                [alert setMessageText: NSLocalizedString(@"Log Could Not Be Saved", "Save log alert panel -> title")];
+                [alert setInformativeText: [NSString stringWithFormat:
+                                            NSLocalizedString(@"There was a problem creating the file \"%@\".",
+                                                              "Save log alert panel -> message"), [[[panel URL] path] lastPathComponent]]];
+                [alert setAlertStyle: NSWarningAlertStyle];
+                
+                [alert runModal];
+                [alert release];
+            }
         }
-    }
-    
-    [messages release];
+    }];
 }
 
 @end
@@ -455,16 +442,11 @@
     const NSInteger level = [[NSUserDefaults standardUserDefaults] integerForKey: @"MessageLevel"];
     NSString * filterString = [fFilterField stringValue];
     
-    NSMutableArray * tempMessages = [NSMutableArray arrayWithCapacity: [fMessages count]]; //rough guess
+    NSIndexSet * indexes = [fMessages indexesOfObjectsWithOptions: NSEnumerationConcurrent passingTest: ^BOOL(id message, NSUInteger idx, BOOL * stop) {
+        return [[(NSDictionary *)message objectForKey: @"Level"] integerValue] <= level && [self shouldIncludeMessageForFilter: filterString message: message];
+    }];
     
-    for (NSDictionary * message in fMessages)
-    {
-        if ([[message objectForKey: @"Level"] integerValue] <= level
-            && [self shouldIncludeMessageForFilter: filterString message: message])
-            [tempMessages addObject: message];
-    }
-    
-    [tempMessages sortUsingDescriptors: [fMessageTable sortDescriptors]];
+    NSArray * tempMessages = [[fMessages objectsAtIndexes: indexes] sortedArrayUsingDescriptors: [fMessageTable sortDescriptors]];
     
     const BOOL onLion = [NSApp isOnLionOrBetter];
     
