@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: PrefsController.m 12693 2011-08-17 01:49:55Z livings124 $
+ * $Id: PrefsController.m 13251 2012-03-13 02:52:11Z livings124 $
  *
- * Copyright (c) 2005-2011 Transmission authors and contributors
+ * Copyright (c) 2005-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,13 +27,13 @@
 #import "BlocklistScheduler.h"
 #import "PortChecker.h"
 #import "BonjourController.h"
-#import "NSApplicationAdditions.h"
 #import "NSStringAdditions.h"
 #import "UKKQueue.h"
 
 #import "transmission.h"
 #import "utils.h"
 
+#import <Growl/Growl.h>
 #import <Sparkle/Sparkle.h>
 
 #define DOWNLOAD_FOLDER     0
@@ -59,11 +59,6 @@
 
 - (void) setPrefView: (id) sender;
 
-- (void) folderSheetClosed: (NSOpenPanel *) openPanel returnCode: (int) code contextInfo: (void *) info;
-- (void) incompleteFolderSheetClosed: (NSOpenPanel *) openPanel returnCode: (int) code contextInfo: (void *) info;
-- (void) importFolderSheetClosed: (NSOpenPanel *) openPanel returnCode: (int) code contextInfo: (void *) info;
-- (void) doneScriptSheetClosed: (NSOpenPanel *) openPanel returnCode: (int) code contextInfo: (void *) info;
-
 - (void) setKeychainPassword: (const char *) password forService: (const char *) service username: (const char *) username;
 
 @end
@@ -143,6 +138,9 @@ tr_session * fHandle;
             [fDefaults removeObjectForKey: @"CheckForUpdates"];
         }
         
+        //set built-in Growl
+        [GrowlApplicationBridge setShouldUseBuiltInNotifications: [fDefaults boolForKey: @"DisplayNotifications"]];
+        
         [self setAutoUpdateToBeta: nil];
     }
     
@@ -182,6 +180,11 @@ tr_session * fHandle;
     
     [self setPrefView: nil];
     
+    [fBuiltInGrowlButton setState: [fDefaults boolForKey: @"DisplayNotifications"]];
+    const BOOL growlRunning = [GrowlApplicationBridge isGrowlRunning];
+    [fBuiltInGrowlButton setHidden: growlRunning];
+    [fGrowlInstalledField setHidden: !growlRunning];
+    
     //set download folder
     [fFolderPopUp selectItemAtIndex: [fDefaults boolForKey: @"DownloadLocationConstant"] ? DOWNLOAD_FOLDER : DOWNLOAD_TORRENT];
     
@@ -541,20 +544,7 @@ tr_session * fHandle;
         NSDate * updatedDate = [fDefaults objectForKey: @"BlocklistNewLastUpdateSuccess"];
         
         if (updatedDate)
-        {
-            if ([NSApp isOnSnowLeopardOrBetter])
-                updatedDateString = [NSDateFormatter localizedStringFromDate: updatedDate dateStyle: NSDateFormatterFullStyle
-                                        timeStyle: NSDateFormatterShortStyle];
-            else
-            {
-                NSDateFormatter * dateFormatter = [[NSDateFormatter alloc] init];
-                [dateFormatter setDateStyle: NSDateFormatterFullStyle];
-                [dateFormatter setTimeStyle: NSDateFormatterShortStyle];
-                
-                updatedDateString = [dateFormatter stringFromDate: updatedDate];
-                [dateFormatter release];
-            }
-        }
+            updatedDateString = [NSDateFormatter localizedStringFromDate: updatedDate dateStyle: NSDateFormatterFullStyle timeStyle: NSDateFormatterShortStyle];
         else
             updatedDateString = NSLocalizedString(@"N/A", "Prefs -> blocklist -> message");
     }
@@ -743,6 +733,13 @@ tr_session * fHandle;
     [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateUI" object: self];
 }
 
+- (IBAction) setBuiltInGrowlEnabled: (id) sender
+{
+    const BOOL enable = [sender state] == NSOnState;
+    [fDefaults setBool: enable forKey: @"DisplayNotifications"];
+    [GrowlApplicationBridge setShouldUseBuiltInNotifications: enable];
+}
+
 - (void) resetWarnings: (id) sender
 {
     [fDefaults removeObjectForKey: @"WarningDuplicate"];
@@ -818,10 +815,24 @@ tr_session * fHandle;
     [panel setCanChooseFiles: NO];
     [panel setCanChooseDirectories: YES];
     [panel setCanCreateDirectories: YES];
-
-    [panel beginSheetForDirectory: nil file: nil types: nil
-        modalForWindow: [self window] modalDelegate: self didEndSelector:
-        @selector(folderSheetClosed:returnCode:contextInfo:) contextInfo: nil];
+    
+    [panel beginSheetModalForWindow: [self window] completionHandler: ^(NSInteger result) {
+        if (result == NSFileHandlingPanelOKButton)
+        {
+            [fFolderPopUp selectItemAtIndex: DOWNLOAD_FOLDER];
+            
+            NSString * folder = [[[panel URLs] objectAtIndex: 0] path];
+            [fDefaults setObject: folder forKey: @"DownloadFolder"];
+            [fDefaults setObject: @"Constant" forKey: @"DownloadChoice"];
+            
+            tr_sessionSetDownloadDir(fHandle, [folder UTF8String]);
+        }
+        else
+        {
+            //reset if cancelled
+            [fFolderPopUp selectItemAtIndex: [fDefaults boolForKey: @"DownloadLocationConstant"] ? DOWNLOAD_FOLDER : DOWNLOAD_TORRENT];
+        }
+    }];
 }
 
 - (void) incompleteFolderSheetShow: (id) sender
@@ -833,10 +844,17 @@ tr_session * fHandle;
     [panel setCanChooseFiles: NO];
     [panel setCanChooseDirectories: YES];
     [panel setCanCreateDirectories: YES];
-
-    [panel beginSheetForDirectory: nil file: nil types: nil
-        modalForWindow: [self window] modalDelegate: self didEndSelector:
-        @selector(incompleteFolderSheetClosed:returnCode:contextInfo:) contextInfo: nil];
+    
+    [panel beginSheetModalForWindow: [self window] completionHandler: ^(NSInteger result) {
+        if (result == NSFileHandlingPanelOKButton)
+        {
+            NSString * folder = [[[panel URLs] objectAtIndex: 0] path];
+            [fDefaults setObject: folder forKey: @"IncompleteDownloadFolder"];
+            
+            tr_sessionSetIncompleteDir(fHandle, [folder UTF8String]);
+        }
+        [fIncompleteFolderPopUp selectItemAtIndex: 0];
+    }];
 }
 
 - (void) doneScriptSheetShow:(id)sender
@@ -849,9 +867,19 @@ tr_session * fHandle;
     [panel setCanChooseDirectories: NO];
     [panel setCanCreateDirectories: NO];
     
-    [panel beginSheetForDirectory: nil file: nil types: nil
-                   modalForWindow: [self window] modalDelegate: self didEndSelector:
-     @selector(doneScriptSheetClosed:returnCode:contextInfo:) contextInfo: nil];
+    [panel beginSheetModalForWindow: [self window] completionHandler: ^(NSInteger result) {
+        if (result == NSFileHandlingPanelOKButton)
+        {
+            NSString * filePath = [[[panel URLs] objectAtIndex: 0] path];
+            
+            [fDefaults setObject: filePath forKey: @"DoneScriptPath"];
+            tr_sessionSetTorrentDoneScript(fHandle, [filePath UTF8String]);
+            
+            [fDefaults setBool: YES forKey: @"DoneScriptEnabled"];
+            tr_sessionSetTorrentDoneScriptEnabled(fHandle, YES);
+        }
+        [fDoneScriptPopUp selectItemAtIndex: 0];
+    }];
 }
 
 - (void) setUseIncompleteFolder: (id) sender
@@ -902,9 +930,25 @@ tr_session * fHandle;
     [panel setCanChooseDirectories: YES];
     [panel setCanCreateDirectories: YES];
 
-    [panel beginSheetForDirectory: nil file: nil types: nil
-        modalForWindow: [self window] modalDelegate: self didEndSelector:
-        @selector(importFolderSheetClosed:returnCode:contextInfo:) contextInfo: nil];
+    [panel beginSheetModalForWindow: [self window] completionHandler: ^(NSInteger result) {
+        NSString * path = [fDefaults stringForKey: @"AutoImportDirectory"];
+        if (result == NSFileHandlingPanelOKButton)
+        {
+            UKKQueue * sharedQueue = [UKKQueue sharedFileWatcher];
+            if (path)
+                [sharedQueue removePathFromQueue: [path stringByExpandingTildeInPath]];
+            
+            path = [[[panel URLs] objectAtIndex: 0] path];
+            [fDefaults setObject: path forKey: @"AutoImportDirectory"];
+            [sharedQueue addPath: [path stringByExpandingTildeInPath]];
+            
+            [[NSNotificationCenter defaultCenter] postNotificationName: @"AutoImportSettingChange" object: self];
+        }
+        else if (!path)
+            [fDefaults setBool: NO forKey: @"AutoImport"];
+        
+        [fImportFolderPopUp selectItemAtIndex: 0];
+    }];
 }
 
 - (void) setAutoSize: (id) sender
@@ -1370,7 +1414,7 @@ tr_session * fHandle;
         return;
     
     NSRect windowRect = [window frame];
-    float difference = ([view frame].size.height - [[window contentView] frame].size.height) * [window userSpaceScaleFactor];
+    const CGFloat difference = (NSHeight([view frame]) - NSHeight([[window contentView] frame])) * [window userSpaceScaleFactor];
     windowRect.origin.y -= difference;
     windowRect.size.height += difference;
     
@@ -1395,80 +1439,6 @@ tr_session * fHandle;
     }
 }
 
-- (void) folderSheetClosed: (NSOpenPanel *) openPanel returnCode: (int) code contextInfo: (void *) info
-{
-    if (code == NSOKButton)
-    {
-        [fFolderPopUp selectItemAtIndex: DOWNLOAD_FOLDER];
-        
-        NSString * folder = [[openPanel filenames] objectAtIndex: 0];
-        [fDefaults setObject: folder forKey: @"DownloadFolder"];
-        [fDefaults setObject: @"Constant" forKey: @"DownloadChoice"];
-        
-        tr_sessionSetDownloadDir(fHandle, [folder UTF8String]);
-    }
-    else
-    {
-        //reset if cancelled
-        [fFolderPopUp selectItemAtIndex: [fDefaults boolForKey: @"DownloadLocationConstant"] ? DOWNLOAD_FOLDER : DOWNLOAD_TORRENT];
-    }
-}
-
-- (void) incompleteFolderSheetClosed: (NSOpenPanel *) openPanel returnCode: (int) code contextInfo: (void *) info
-{
-    if (code == NSOKButton)
-    {
-        NSString * folder = [[openPanel filenames] objectAtIndex: 0];
-        [fDefaults setObject: folder forKey: @"IncompleteDownloadFolder"];
-        
-        tr_sessionSetIncompleteDir(fHandle, [folder UTF8String]);
-    }
-    [fIncompleteFolderPopUp selectItemAtIndex: 0];
-}
-
-- (void) importFolderSheetClosed: (NSOpenPanel *) openPanel returnCode: (int) code contextInfo: (void *) info
-{
-    NSString * path = [fDefaults stringForKey: @"AutoImportDirectory"];
-    if (code == NSOKButton)
-    {
-        UKKQueue * sharedQueue = [UKKQueue sharedFileWatcher];
-        if (path)
-            [sharedQueue removePathFromQueue: [path stringByExpandingTildeInPath]];
-        
-        path = [[openPanel filenames] objectAtIndex: 0];
-        [fDefaults setObject: path forKey: @"AutoImportDirectory"];
-        [sharedQueue addPath: [path stringByExpandingTildeInPath]];
-        
-        [[NSNotificationCenter defaultCenter] postNotificationName: @"AutoImportSettingChange" object: self];
-    }
-    else if (!path)
-        [fDefaults setBool: NO forKey: @"AutoImport"];
-    
-    [fImportFolderPopUp selectItemAtIndex: 0];
-}
-
-- (void) doneScriptSheetClosed: (NSOpenPanel *) openPanel returnCode: (int) code contextInfo: (void *) info
-{
-    if (code == NSOKButton)
-    {
-        NSString * filePath = [[openPanel filenames] objectAtIndex: 0];
-        
-        if ([[NSFileManager defaultManager] fileExistsAtPath: filePath])  // script file exists
-        {
-            [fDefaults setObject: filePath forKey: @"DoneScriptPath"];
-            [fDefaults setBool: YES forKey: @"DoneScriptEnabled"];
-        }
-        else // script file doesn't exist so don't enable
-        {
-            [fDefaults setObject: nil forKey:@"DoneScriptPath"];
-            [fDefaults setBool: NO forKey: @"DoneScriptEnabled"];
-        }
-        tr_sessionSetTorrentDoneScript(fHandle, [[fDefaults stringForKey:@"DoneScriptPath"] UTF8String]);
-        tr_sessionSetTorrentDoneScriptEnabled(fHandle, [fDefaults boolForKey:@"DoneScriptEnabled"]);
-    }
-    [fDoneScriptPopUp selectItemAtIndex: 0];
-}
-
 - (void) setKeychainPassword: (const char *) password forService: (const char *) service username: (const char *) username
 {
     SecKeychainItemRef item = NULL;
