@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: StatsWindowController.m 11419 2010-11-14 21:51:56Z livings124 $
+ * $Id: StatsWindowController.m 13251 2012-03-13 02:52:11Z livings124 $
  *
- * Copyright (c) 2007-2010 Transmission authors and contributors
+ * Copyright (c) 2007-2012 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -126,7 +126,7 @@ tr_session * fLib;
 {
     [fTimer invalidate];
     
-    [fStatsWindowInstance release];
+    [fStatsWindowInstance autorelease];
     fStatsWindowInstance = nil;
 }
 
@@ -196,8 +196,8 @@ tr_session * fLib;
     if (statsAll.sessionCount == 1)
         [fNumOpenedField setStringValue: NSLocalizedString(@"1 time", "stats window -> times opened")];
     else
-        [fNumOpenedField setStringValue: [NSString stringWithFormat: NSLocalizedString(@"%llu times", "stats window -> times opened"),
-                                            statsAll.sessionCount]];
+        [fNumOpenedField setStringValue: [NSString stringWithFormat: NSLocalizedString(@"%@ times", "stats window -> times opened"),
+                                            [NSString formattedUInteger: statsAll.sessionCount]]];
 }
 
 - (void) performResetStats
