@@ -7,7 +7,7 @@
  *
  * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
  *
- * $Id: file-tree.cc 11405 2010-11-13 22:07:19Z charles $
+ * $Id: file-tree.cc 12697 2011-08-20 05:19:27Z jordan $
  */
 
 #include <cassert>
@@ -420,7 +420,7 @@ FileTreeModel :: addFile( int                   index,
 {
     FileTreeItem * i( rootItem );
 
-    foreach( QString token, filename.split( "/" ) )
+    foreach( QString token, filename.split( QChar::fromAscii('/') ) )
     {
         FileTreeItem * child( i->child( token ) );
         if( !child )
@@ -696,7 +696,7 @@ FileTreeView :: eventFilter( QObject * o, QEvent * event )
     }
 
     // handle using the keyboard to toggle the
-    // wanted/unwanted state or the file priority 
+    // wanted/unwanted state or the file priority
     else if( event->type() == QEvent::KeyPress )
     {
         switch( dynamic_cast<QKeyEvent*>(event)->key() )
