@@ -7,7 +7,7 @@
  *
  * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
  *
- * $Id: filterbar.cc 11423 2010-11-15 15:19:57Z charles $
+ * $Id: filterbar.cc 12607 2011-08-01 22:24:24Z jordan $
  */
 
 #include <QString>
@@ -247,10 +247,6 @@ FilterBar :: createActivityCombo( )
     row->setData( FilterMode::SHOW_FINISHED, ActivityRole );
     model->appendRow( row );
 
-    row = new QStandardItem( blankIcon, tr( "Queued" ) );
-    row->setData( FilterMode::SHOW_QUEUED, ActivityRole );
-    model->appendRow( row );
-
     row = new QStandardItem( QIcon::fromTheme( "view-refresh", blankIcon ), tr( "Verifying" ) );
     row->setData( FilterMode::SHOW_VERIFYING, ActivityRole );
     model->appendRow( row );
@@ -265,7 +261,7 @@ FilterBar :: createActivityCombo( )
 
 /****
 *****
-*****  
+*****
 *****
 ****/
 
@@ -309,13 +305,18 @@ FilterBar :: refreshTrackers( )
             break;
         const Torrent * tor = index.data( TorrentModel::TorrentRole ).value<const Torrent*>();
         const QStringList trackers = tor->trackers( );
-        QSet<QString> torrentHosts;
-        foreach( QString tracker, trackers )
-            torrentHosts.insert( Favicons::getHost( QUrl( tracker ) ) );
-        foreach( QString host, torrentHosts ) {
-            newHosts.insert( host );
-            ++torrentsPerHost[host];
+        QSet<QString> torrentNames;
+        foreach( QString tracker, trackers ) {
+            const QString host = Favicons::getHost( QUrl( tracker ) );
+            if( host.isEmpty( ) )
+                qWarning() << "torrent" << qPrintable(tor->name()) << "has an invalid announce URL:" << tracker;
+            else {
+                newHosts.insert( host );
+                torrentNames.insert( readableHostName( host ) );
+            }
         }
+        foreach( QString name, torrentNames )
+            ++torrentsPerHost[ name ];
     }
 
     // update the "All" row
@@ -326,7 +327,7 @@ FilterBar :: refreshTrackers( )
     {
         const QString name = readableHostName( host );
         QStandardItem * row = myTrackerModel->findItems(name).front();
-        row->setData( getCountString(torrentsPerHost[host]), TorrentCountRole );
+        row->setData( getCountString(torrentsPerHost[name]), TorrentCountRole );
         row->setData( favicons.findFromHost(host), Qt::DecorationRole );
     }
 
@@ -344,14 +345,19 @@ FilterBar :: refreshTrackers( )
     {
         const QString name = readableHostName( host );
 
+        if( !myTrackerModel->findItems(name).isEmpty() )
+            continue;
+
         // find the sorted position to add this row
         int i = firstTrackerRow;
-        for( int n=myTrackerModel->rowCount(); i<n; ++i )
-            if( myTrackerModel->index(i,0).data(Qt::DisplayRole).toString() > name )
+        for( int n=myTrackerModel->rowCount(); i<n; ++i ) {
+            const QString rowName = myTrackerModel->index(i,0).data(Qt::DisplayRole).toString();
+            if( rowName >= name )
                 break;
+        }
 
         // add the row
-        QStandardItem * row = new QStandardItem( favicons.findFromHost( host ), readableHostName( host ) );
+        QStandardItem * row = new QStandardItem( favicons.findFromHost( host ), name );
         row->setData( getCountString(torrentsPerHost[host]), TorrentCountRole );
         row->setData( favicons.findFromHost(host), Qt::DecorationRole );
         row->setData( host, TrackerRole );
@@ -385,7 +391,7 @@ FilterBar :: createTrackerCombo( QStandardItemModel * model )
 
 /****
 *****
-*****  
+*****
 *****
 ****/
 
@@ -413,7 +419,7 @@ FilterBar :: FilterBar( Prefs& prefs, TorrentModel& torrents, TorrentFilter& fil
     myTrackerCombo = createTrackerCombo( myTrackerModel );
     h->addWidget( myTrackerCombo, 1 );
     h->addSpacing( hmargin*2 );
-    
+
     myLineEdit = new QLineEdit( this );
     h->addWidget( myLineEdit );
     connect( myLineEdit, SIGNAL(textChanged(QString)), this, SLOT(onTextChanged(QString)));
@@ -473,9 +479,10 @@ FilterBar :: refreshPref( int key )
 
         case Prefs :: FILTER_TRACKERS: {
             const QString tracker = myPrefs.getString( key );
-            QModelIndexList indices = myTrackerModel->match( myTrackerModel->index(0,0), TrackerRole, tracker, 1, Qt::MatchFixedString );
-            if( !indices.isEmpty( )  )
-                myTrackerCombo->setCurrentIndex( indices.first().row() );
+            const QString name = readableHostName( tracker );
+            QList<QStandardItem*> rows = myTrackerModel->findItems(name);
+            if( !rows.isEmpty() )
+                myTrackerCombo->setCurrentIndex( rows.front()->row() );
             else { // hm, we don't seem to have this tracker anymore...
                 const bool isBootstrapping = myTrackerModel->rowCount( ) <= 2;
                 if( !isBootstrapping )
@@ -504,10 +511,14 @@ FilterBar :: onTrackerIndexChanged( int i )
     {
         QString str;
         const bool isTracker = !myTrackerCombo->itemData(i,TrackerRole).toString().isEmpty();
-        if( isTracker )
-            str = myTrackerCombo->itemData(i,TrackerRole).toString();
-        else // show all
+        if( !isTracker ) // show all
             str = "";
+        else {
+            str = myTrackerCombo->itemData(i,TrackerRole).toString();
+            const int pos = str.lastIndexOf( '.' );
+            if( pos >= 0 )
+              str.truncate( pos+1 );
+        }
         myPrefs.set( Prefs::FILTER_TRACKERS, str );
     }
 }
