@@ -7,7 +7,7 @@
  *
  * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
  *
- * $Id: session.cc 11616 2010-12-31 19:44:51Z charles $
+ * $Id: session.cc 12661 2011-08-09 13:35:44Z jordan $
  */
 
 #include <cassert>
@@ -151,6 +151,8 @@ Session :: updatePref( int key )
         case Prefs :: BLOCKLIST_URL:
         case Prefs :: DHT_ENABLED:
         case Prefs :: DOWNLOAD_DIR:
+        case Prefs :: DOWNLOAD_QUEUE_ENABLED:
+        case Prefs :: DOWNLOAD_QUEUE_SIZE:
         case Prefs :: DSPEED:
         case Prefs :: DSPEED_ENABLED:
         case Prefs :: IDLE_LIMIT:
@@ -162,6 +164,7 @@ Session :: updatePref( int key )
         case Prefs :: PEER_LIMIT_TORRENT:
         case Prefs :: PEER_PORT:
         case Prefs :: PEER_PORT_RANDOM_ON_START:
+        case Prefs :: QUEUE_STALLED_MINUTES:
         case Prefs :: PEX_ENABLED:
         case Prefs :: PORT_FORWARDING:
         case Prefs :: SCRIPT_TORRENT_DONE_ENABLED:
@@ -170,6 +173,7 @@ Session :: updatePref( int key )
         case Prefs :: TRASH_ORIGINAL:
         case Prefs :: USPEED:
         case Prefs :: USPEED_ENABLED:
+        case Prefs :: UTP_ENABLED:
             sessionSet( myPrefs.keyStr(key), myPrefs.variant(key) );
             break;
 
@@ -306,37 +310,6 @@ Session :: restart( )
     start( );
 }
 
-static void
-curlConfigFunc( tr_session * session UNUSED, void * vcurl, const char * destination )
-{
-    CURL * easy = vcurl;
-    const QUrl url( destination );
-    const QNetworkProxyQuery query( url );
-    QList<QNetworkProxy> proxyList = QNetworkProxyFactory :: systemProxyForQuery( query );
-
-    foreach( QNetworkProxy proxy, proxyList )
-    {
-        long type = -1;
-
-        switch( proxy.type( ) ) {
-            case QNetworkProxy::HttpProxy: type = CURLPROXY_HTTP; break;
-            case QNetworkProxy::Socks5Proxy: type = CURLPROXY_SOCKS5; break;
-            default: break;
-        }
-
-        if( type != -1 ) {
-            curl_easy_setopt( easy, CURLOPT_PROXY, proxy.hostName().toUtf8().data() );
-            curl_easy_setopt( easy, CURLOPT_PROXYPORT, long(proxy.port()) );
-            curl_easy_setopt( easy, CURLOPT_PROXYTYPE, type );
-            const QString user = proxy.user();
-            const QString pass = proxy.password();
-            if( !user.isEmpty() && !pass.isEmpty() )
-                curl_easy_setopt( easy, CURLOPT_PROXYUSERPWD, (user+":"+pass).toUtf8().data() );
-            return;
-        }
-    }
-}
-
 void
 Session :: start( )
 {
@@ -360,7 +333,6 @@ Session :: start( )
         tr_bencInitDict( &settings, 0 );
         tr_sessionLoadSettings( &settings, myConfigDir.toUtf8().constData(), "qt" );
         mySession = tr_sessionInit( "qt", myConfigDir.toUtf8().constData(), true, &settings );
-        tr_sessionSetWebConfigFunc( mySession, curlConfigFunc );
         tr_bencFree( &settings );
 
         tr_ctor * ctor = tr_ctorNew( mySession );
@@ -567,17 +539,13 @@ Session :: sendTorrentRequest( const char * request, const QSet<int>& ids )
     refreshTorrents( ids );
 }
 
-void
-Session :: pauseTorrents( const QSet<int>& ids )
-{
-    sendTorrentRequest( "torrent-stop", ids );
-}
-
-void
-Session :: startTorrents( const QSet<int>& ids )
-{
-    sendTorrentRequest( "torrent-start", ids );
-}
+void Session :: pauseTorrents    ( const QSet<int>& ids ) { sendTorrentRequest( "torrent-stop", ids ); }
+void Session :: startTorrents    ( const QSet<int>& ids ) { sendTorrentRequest( "torrent-start", ids ); } 
+void Session :: startTorrentsNow ( const QSet<int>& ids ) { sendTorrentRequest( "torrent-start-now", ids ); }
+void Session :: queueMoveTop     ( const QSet<int>& ids ) { sendTorrentRequest( "queue-move-top", ids ); } 
+void Session :: queueMoveUp      ( const QSet<int>& ids ) { sendTorrentRequest( "queue-move-up", ids ); } 
+void Session :: queueMoveDown    ( const QSet<int>& ids ) { sendTorrentRequest( "queue-move-down", ids ); } 
+void Session :: queueMoveBottom  ( const QSet<int>& ids ) { sendTorrentRequest( "queue-move-bottom", ids ); } 
 
 void
 Session :: refreshActiveTorrents( )
@@ -798,7 +766,7 @@ Session :: parseResponse( const char * json, size_t jsonLength )
                 }
 
                 case TAG_PORT_TEST: {
-                    tr_bool isOpen = 0;
+                    bool isOpen = 0;
                     if( tr_bencDictFindDict( &top, "arguments", &args ) )
                         tr_bencDictFindBool( args, "port-is-open", &isOpen );
                     emit portTested( (bool)isOpen );
@@ -916,7 +884,7 @@ Session :: updateInfo( tr_benc * d )
                 break;
             }
             case QVariant :: Bool: {
-                tr_bool val;
+                bool val;
                 if( tr_bencGetBool( b, &val ) )
                     myPrefs.set( i, (bool)val );
                 break;
@@ -934,7 +902,7 @@ Session :: updateInfo( tr_benc * d )
         }
     }
 
-    tr_bool b;
+    bool b;
     double x;
     if( tr_bencDictFindBool( d, "seedRatioLimited", &b ) )
         myPrefs.set( Prefs::RATIO_ENABLED, b ? true : false );
