@@ -7,13 +7,13 @@
  *
  * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
  *
- * $Id: torrent-delegate.h 11092 2010-08-01 20:36:13Z charles $
+ * $Id: torrent-delegate.h 13244 2012-03-04 13:15:43Z jordan $
  */
 
 #ifndef QTR_TORRENT_DELEGATE_H
 #define QTR_TORRENT_DELEGATE_H
 
-#include <QItemDelegate>
+#include <QStyledItemDelegate>
 #include <QSize>
 
 class QStyleOptionProgressBarV2;
@@ -22,10 +22,14 @@ class QStyle;
 class Session;
 class Torrent;
 
-class TorrentDelegate: public QItemDelegate
+class TorrentDelegate: public QStyledItemDelegate
 {
         Q_OBJECT
 
+    public:
+      static QColor blueBrush, greenBrush;
+      static QColor blueBack,  greenBack;
+
     protected:
         QStyleOptionProgressBarV2 * myProgressBarStyle;
 
@@ -38,6 +42,7 @@ class TorrentDelegate: public QItemDelegate
     protected:
         QSize margin( const QStyle& style ) const;
         virtual QSize sizeHint( const QStyleOptionViewItem&, const Torrent& ) const;
+        virtual void setProgressBarPercentDone( const QStyleOptionViewItem& option, const Torrent& ) const;
         virtual void drawTorrent( QPainter* painter, const QStyleOptionViewItem& option, const Torrent& ) const;
 
     public:
@@ -46,6 +51,7 @@ class TorrentDelegate: public QItemDelegate
 
         QSize sizeHint(const QStyleOptionViewItem& option, const QModelIndex& index) const;
         void paint(QPainter* painter, const QStyleOptionViewItem& option, const QModelIndex& index) const;
+
 };
 
 #endif
