@@ -7,7 +7,7 @@
  *
  * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
  *
- * $Id: utils.h 11092 2010-08-01 20:36:13Z charles $
+ * $Id: utils.h 12697 2011-08-20 05:19:27Z jordan $
  */
 
 #ifndef QTR_UTILS
@@ -17,6 +17,8 @@
 #include <QObject>
 #include <QIcon>
 
+#include <cctype> // isxdigit()
+
 #include "speed.h"
 
 class Utils: public QObject
@@ -34,15 +36,26 @@ class Utils: public QObject
         // meh
         static void toStderr( const QString& qstr );
 
-        /// 
+        ///
         /// URLs
         ///
 
-        static bool isMagnetLink( const QString& s ) { return s.startsWith( "magnet:?" ); }
+        static bool isMagnetLink( const QString& s ) { return s.startsWith( QString::fromAscii( "magnet:?" ) ); }
 
-        static bool isURL( const QString& s ) { return s.startsWith( "http://" )
-                                                    || s.startsWith( "https://" )
-                                                    || s.startsWith( "ftp://" ); }
+        static bool isHexHashcode( const QString& s )
+        {
+            if( s.length() != 40 ) return false;
+            foreach( QChar ch, s ) if( !isxdigit( ch.toAscii() ) ) return false;
+            return true;
+        }
+
+        static bool isUriWithSupportedScheme( const QString& s )
+        {
+            static const QString ftp = QString::fromAscii( "ftp://" );
+            static const QString http = QString::fromAscii( "http://" );
+            static const QString https = QString::fromAscii( "https://" );
+            return s.startsWith(http) || s.startsWith(https) || s.startsWith(ftp);
+        }
 
 };
 
